@@ -63,24 +63,27 @@
 #include "i2c/i2c_bus_core.h"
 #include "i2c/i2c_do_io.h"
 
-#include "ddc/ddc_try_stats.h"
-
 #include "adl/adl_shim.h"
 
 #include "usb/usb_displays.h"
 
 #include "ddc/ddc_displays.h"
+#include "ddc/ddc_dynamic_features.h"
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_output.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_read_capabilities.h"
 #include "ddc/ddc_services.h"
+#include "ddc/ddc_try_stats.h"
 #include "ddc/ddc_vcp_version.h"
 #include "ddc/ddc_vcp.h"
+
 #include "cmdline/cmd_parser_aux.h"    // for parse_feature_id_or_subset(), should it be elsewhere?
 #include "cmdline/cmd_parser.h"
 #include "cmdline/parsed_cmd.h"
 
+#include "app_common/app_dynamic_features.h"    // future
+
 #include "app_ddcutil/app_dumpload.h"
 #include "app_ddcutil/app_getvcp.h"
 #include "app_ddcutil/app_setvcp.h"
@@ -136,6 +139,8 @@ void report_stats(DDCA_Stats_Type stats) {
 //       these actions should be separated
 Parsed_Capabilities *
 perform_get_capabilities_by_display_handle(Display_Handle * dh) {
+   FILE * fout = stdout;
+   FILE * ferr = stderr;
    bool debug = false;
    Parsed_Capabilities * pcap = NULL;
    char * capabilities_string;
@@ -147,14 +152,14 @@ perform_get_capabilities_by_display_handle(Display_Handle * dh) {
       switch(psc) {
       case DDCRC_REPORTED_UNSUPPORTED:       // should not happen
       case DDCRC_DETERMINED_UNSUPPORTED:
-         printf("Unsupported request\n");
+         f0printf(ferr, "Unsupported request\n");
          break;
       case DDCRC_RETRIES:
-         f0printf(FOUT, "Unable to get capabilities for monitor on %s.  Maximum DDC retries exceeded.\n",
+         f0printf(ferr, "Unable to get capabilities for monitor on %s.  Maximum DDC retries exceeded.\n",
                  dh_repr(dh));
          break;
       default:
-         f0printf(FOUT, "(%s) !!! Unable to get capabilities for monitor on %s\n",
+         f0printf(ferr, "(%s) !!! Unable to get capabilities for monitor on %s\n",
                 __func__, dh_repr(dh));
          DBGMSG("Unexpected status code: %s", psc_desc(psc));
       }
@@ -167,7 +172,7 @@ perform_get_capabilities_by_display_handle(Display_Handle * dh) {
       pcap = parse_capabilities_string(capabilities_string);
       DDCA_Output_Level output_level = get_output_level();
       if (output_level <= DDCA_OL_TERSE) {
-         f0printf(FOUT,
+         f0printf(fout,
                   "%s capabilities string: %s\n",
                   (dh->dref->io_path.io_mode == DDCA_IO_USB) ? "Synthesized unparsed" : "Unparsed",
                   capabilities_string);
@@ -176,7 +181,10 @@ perform_get_capabilities_by_display_handle(Display_Handle * dh) {
          if (dh->dref->io_path.io_mode == DDCA_IO_USB)
             pcap->raw_value_synthesized = true;
          // report_parsed_capabilities(pcap, dh->dref->io_path.io_mode);    // io_mode no longer needed
-         report_parsed_capabilities(pcap);
+         report_parsed_capabilities(
+               pcap,
+               dh->dref->mmid,
+               0);
          // free_parsed_capabilities(pcap);
       }
    }
@@ -187,14 +195,15 @@ perform_get_capabilities_by_display_handle(Display_Handle * dh) {
 
 void probe_display_by_dh(Display_Handle * dh)
 {
+   FILE * fout = stdout;
    bool debug = false;
    DBGMSF(debug, "Starting. dh=%s", dh_repr(dh));
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
 
-   f0printf(FOUT, "\nMfg id: %s, model: %s, sn: %s\n",
+   f0printf(fout, "\nMfg id: %s, model: %s, sn: %s\n",
                   dh->dref->pedid->mfg_id, dh->dref->pedid->model_name, dh->dref->pedid->serial_ascii);
-   f0printf(FOUT, "\nCapabilities for display on %s\n", dref_short_name_t(dh->dref));
+   f0printf(fout, "\nCapabilities for display on %s\n", dref_short_name_t(dh->dref));
 
    DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
    // not needed, message causes confusing messages if get_vcp_version fails but get_capabilities succeeds
@@ -207,58 +216,64 @@ void probe_display_by_dh(Display_Handle * dh)
 
    // how to pass this information down into app_show_vcp_subset_values_by_display_handle()?
    bool table_reads_possible = parsed_capabilities_may_support_table_commands(pcaps);
-   f0printf(FOUT, "\nMay support table reads:   %s\n", bool_repr(table_reads_possible));
+   f0printf(fout, "\nMay support table reads:   %s\n", bool_repr(table_reads_possible));
 
    // *** VCP Feature Scan ***
    // printf("\n\nScanning all VCP feature codes for display %d\n", dispno);
-   f0printf(FOUT, "\nScanning all VCP feature codes for display %s\n", dh_repr(dh) );
+   f0printf(fout, "\nScanning all VCP feature codes for display %s\n", dh_repr(dh) );
    Byte_Bit_Flags features_seen = bbf_create();
    app_show_vcp_subset_values_by_display_handle(
          dh, VCP_SUBSET_SCAN, FSF_SHOW_UNSUPPORTED, features_seen);
 
    if (pcaps) {
-      f0printf(FOUT, "\n\nComparing declared capabilities to observed features...\n");
+      f0printf(fout, "\n\nComparing declared capabilities to observed features...\n");
       Byte_Bit_Flags features_declared =
             parsed_capabilities_feature_ids(pcaps, /*readable_only=*/true);
       char * s0 = bbf_to_string(features_declared, NULL, 0);
-      f0printf(FOUT, "\nReadable features declared in capabilities string: %s\n", s0);
+      f0printf(fout, "\nReadable features declared in capabilities string: %s\n", s0);
       free(s0);
 
       Byte_Bit_Flags caps_not_seen = bbf_subtract(features_declared, features_seen);
       Byte_Bit_Flags seen_not_caps = bbf_subtract(features_seen, features_declared);
 
-      f0printf(FOUT, "\nMCCS (VCP) version reported by capabilities: %s\n",
+      f0printf(fout, "\nMCCS (VCP) version reported by capabilities: %s\n",
                format_vspec(pcaps->parsed_mccs_version));
-      f0printf(FOUT, "MCCS (VCP) version reported by feature 0xDf: %s\n",
+      f0printf(fout, "MCCS (VCP) version reported by feature 0xDf: %s\n",
                format_vspec(vspec));
       if (!vcp_version_eq(pcaps->parsed_mccs_version, vspec))
-         f0printf(FOUT, "Versions do not match!!!\n");
+         f0printf(fout, "Versions do not match!!!\n");
 
       if (bbf_count_set(caps_not_seen) > 0) {
-         f0printf(FOUT, "\nFeatures declared as readable capabilities but not found by scanning:\n");
+         f0printf(fout, "\nFeatures declared as readable capabilities but not found by scanning:\n");
          for (int code = 0; code < 256; code++) {
             if (bbf_is_set(caps_not_seen, code)) {
                VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid_w_default(code);
                char * feature_name = get_version_sensitive_feature_name(vfte, pcaps->parsed_mccs_version);
-               f0printf(FOUT, "   Feature x%02x - %s\n", code, feature_name);
+               f0printf(fout, "   Feature x%02x - %s\n", code, feature_name);
+               if (vfte->vcp_global_flags & DDCA_SYNTHETIC) {
+                  free_synthetic_vcp_entry(vfte);
+               }
             }
          }
       }
       else
-         f0printf(FOUT, "\nAll readable features declared in capabilities were found by scanning.\n");
+         f0printf(fout, "\nAll readable features declared in capabilities were found by scanning.\n");
 
       if (bbf_count_set(seen_not_caps) > 0) {
-         f0printf(FOUT, "\nFeatures found by scanning but not declared as capabilities:\n");
+         f0printf(fout, "\nFeatures found by scanning but not declared as capabilities:\n");
          for (int code = 0; code < 256; code++) {
             if (bbf_is_set(seen_not_caps, code)) {
                VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid_w_default(code);
                char * feature_name = get_version_sensitive_feature_name(vfte, vspec);
-               f0printf(FOUT, "   Feature x%02x - %s\n", code, feature_name);
+               f0printf(fout, "   Feature x%02x - %s\n", code, feature_name);
+               if (vfte->vcp_global_flags & DDCA_SYNTHETIC) {
+                  free_synthetic_vcp_entry(vfte);
+               }
             }
          }
       }
       else
-         f0printf(FOUT, "\nAll features found by scanning were declared in capabilities.\n");
+         f0printf(fout, "\nAll features found by scanning were declared in capabilities.\n");
 
       bbf_free(features_declared);
       bbf_free(caps_not_seen);
@@ -266,42 +281,44 @@ void probe_display_by_dh(Display_Handle * dh)
       free_parsed_capabilities(pcaps);
    }
    else {
-      f0printf(FOUT, "\n\nUnable to read or parse capabilities.\n");
-      f0printf(FOUT, "Skipping comparison of declared capabilities to observed features\n");
+      f0printf(fout, "\n\nUnable to read or parse capabilities.\n");
+      f0printf(fout, "Skipping comparison of declared capabilities to observed features\n");
    }
    bbf_free(features_seen);
 
 
    puts("");
    // get VCP 0B
-   DDCA_Single_Vcp_Value * valrec;
+   Single_Vcp_Value * valrec;
    int color_temp_increment = 0;
    int color_temp_units = 0;
-   ddc_excp =  get_vcp_value(
-            dh,
-          0x0b,              // color temperature increment,
-          DDCA_NON_TABLE_VCP_VALUE,
-          &valrec);
+   ddc_excp = ddc_get_vcp_value(
+                 dh,
+                 0x0b,              // color temperature increment,
+                 DDCA_NON_TABLE_VCP_VALUE,
+                 &valrec);
    psc = ERRINFO_STATUS(ddc_excp);
    if (psc == 0) {
       if (debug)
-         f0printf(FOUT, "Value returned for feature x0b: %s\n", summarize_single_vcp_value(valrec) );
+         f0printf(fout, "Value returned for feature x0b: %s\n", summarize_single_vcp_value(valrec) );
       color_temp_increment = valrec->val.c.cur_val;
+      free_single_vcp_value(valrec);
 
-      ddc_excp =  get_vcp_value(
-            dh,
-          0x0c,              // color temperature request
-          DDCA_NON_TABLE_VCP_VALUE,
-          &valrec);
+      ddc_excp = ddc_get_vcp_value(
+                    dh,
+                    0x0c,              // color temperature request
+                    DDCA_NON_TABLE_VCP_VALUE,
+                    &valrec);
       psc = ERRINFO_STATUS(ddc_excp);
       if (psc == 0) {
          if (debug)
-            f0printf(FOUT, "Value returned for feature x0c: %s\n", summarize_single_vcp_value(valrec) );
+            f0printf(fout, "Value returned for feature x0c: %s\n", summarize_single_vcp_value(valrec) );
          color_temp_units = valrec->val.c.cur_val;
+         free_single_vcp_value(valrec);
          int color_temp = 3000 + color_temp_units * color_temp_increment;
-         f0printf(FOUT, "Color temperature increment (x0b) = %d degrees Kelvin\n", color_temp_increment);
-         f0printf(FOUT, "Color temperature request   (x0c) = %d\n", color_temp_units);
-         f0printf(FOUT, "Requested color temperature = (3000 deg Kelvin) + %d * (%d degrees Kelvin)"
+         f0printf(fout, "Color temperature increment (x0b) = %d degrees Kelvin\n", color_temp_increment);
+         f0printf(fout, "Color temperature request   (x0c) = %d\n", color_temp_units);
+         f0printf(fout, "Requested color temperature = (3000 deg Kelvin) + %d * (%d degrees Kelvin)"
                " = %d degrees Kelvin\n",
                color_temp_units,
                color_temp_increment,
@@ -309,7 +326,7 @@ void probe_display_by_dh(Display_Handle * dh)
       }
    }
    if (psc != 0) {
-      f0printf(FOUT, "Unable to calculate color temperature from VCP features x0B and x0C\n");
+      f0printf(fout, "Unable to calculate color temperature from VCP features x0B and x0C\n");
       // errinfo_free(ddc_excp);
       ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || report_freed_exceptions);
    }
@@ -321,10 +338,11 @@ void probe_display_by_dh(Display_Handle * dh)
 
 
 void probe_display_by_dref(Display_Ref * dref) {
+   FILE * fout = stdout;
    Display_Handle * dh = NULL;
    Public_Status_Code psc = ddc_open_display(dref, CALLOPT_ERR_MSG, &dh);
    if (psc != 0) {
-      f0printf(FOUT, "Unable to open display %s, status code %s",
+      f0printf(fout, "Unable to open display %s, status code %s",
                      dref_short_name_t(dref), psc_desc(psc) );
    }
    else {
@@ -347,6 +365,9 @@ void probe_display_by_dref(Display_Ref * dref) {
   * @retval  EXIT_FAILURE an error occurred
   */
 int main(int argc, char *argv[]) {
+   FILE * fout = stdout;
+   bool main_debug = false;
+   int main_rc = EXIT_FAILURE;
 
 #ifdef OBSOLETE
    // For aborting out of shared library
@@ -354,7 +375,7 @@ int main(int argc, char *argv[]) {
    int jmprc = setjmp(abort_buf);
    if (jmprc) {
       fprintf(stderr, "Aborting. Internal status code = %d\n", jmprc);
-      exit(EXIT_FAILURE);
+      goto bye;       // main_rc == EXIT_FAILURE
    }
 
    register_jmp_buf(&abort_buf);
@@ -364,11 +385,11 @@ int main(int argc, char *argv[]) {
    init_base_services();  // so tracing related modules are initialized
    Parsed_Cmd * parsed_cmd = parse_command(argc, argv);
    if (!parsed_cmd) {
-      exit(EXIT_FAILURE);
+      goto bye;      // main_rc == EXIT_FAILURE
    }
-   if (parsed_cmd->timestamp_trace)         // timestamps on debug and trace messages?
+   if (parsed_cmd->flags & CMD_FLAG_TIMESTAMP_TRACE)         // timestamps on debug and trace messages?
       dbgtrc_show_time = true;              // extern in core.h
-   report_freed_exceptions = parsed_cmd->report_freed_exceptions;   // extern in core.h
+   report_freed_exceptions = parsed_cmd->flags & CMD_FLAG_REPORT_FREED_EXCP;   // extern in core.h
    set_trace_levels(parsed_cmd->trace);
    if (parsed_cmd->traced_functions) {
       for (int ndx = 0; ndx < ntsa_length(parsed_cmd->traced_functions); ndx++)
@@ -387,7 +408,7 @@ int main(int argc, char *argv[]) {
       if (!ok) {
          fprintf(stderr, "Error loading failure simulation control file %s.\n",
                          parsed_cmd->failsim_control_fn);
-         exit(EXIT_FAILURE);
+         goto bye;      // main_rc == EXIT_FAILURE
       }
       fsim_report_error_table(0);
    }
@@ -397,7 +418,7 @@ int main(int argc, char *argv[]) {
    // overrides setting in init_ddc_services():
    i2c_set_io_strategy(DEFAULT_I2C_IO_STRATEGY);
 
-   set_verify_setvcp(parsed_cmd->verify_setvcp);
+   ddc_set_verify_setvcp(parsed_cmd->flags & CMD_FLAG_VERIFY);
 
 #ifndef HAVE_ADL
    if ( is_module_loaded_using_sysfs("fglrx") ) {
@@ -406,26 +427,24 @@ int main(int argc, char *argv[]) {
    }
 #endif
 
-   int main_rc = EXIT_FAILURE;
-
    Call_Options callopts = CALLOPT_NONE;
-   i2c_force_slave_addr_flag = parsed_cmd->force_slave_addr;
+   i2c_force_slave_addr_flag = parsed_cmd->flags & CMD_FLAG_FORCE_SLAVE_ADDR;
    if (parsed_cmd->flags & CMD_FLAG_FORCE)
       callopts |= CALLOPT_FORCE;
 
    set_output_level(parsed_cmd->output_level);
-   report_ddc_errors = parsed_cmd->ddcdata;
+   enable_report_ddc_errors( parsed_cmd->flags & CMD_FLAG_DDCDATA );
    // TMI:
    // if (show_recoverable_errors)
    //    parsed_cmd->stats = true;
 
    if (parsed_cmd->output_level >= DDCA_OL_VERBOSE) {
       show_reporting();
-      f0printf( FOUT, "%.*s%-*s%s\n",
+      f0printf( fout, "%.*s%-*s%s\n",
                 0,"",
                 28, "Force I2C slave address:",
                 bool_repr(i2c_force_slave_addr_flag));
-      f0puts("\n", FOUT);
+      f0puts("\n", fout);
    }
 
    // n. MAX_MAX_TRIES checked during command line parsing
@@ -458,26 +477,36 @@ int main(int argc, char *argv[]) {
       set_sleep_strategy(parsed_cmd->sleep_strategy);
 
    int threshold = DISPLAY_CHECK_ASYNC_NEVER;
-   if (parsed_cmd->async)
+   if (parsed_cmd->flags & CMD_FLAG_ASYNC)
       threshold = DISPLAY_CHECK_ASYNC_THRESHOLD;
    ddc_set_async_threshold(threshold);
 
+   main_rc = EXIT_SUCCESS;     // from now on assume success;
+
    if (parsed_cmd->cmd_id == CMDID_LISTVCP) {
       vcp_list_feature_codes(stdout);
       main_rc = EXIT_SUCCESS;
    }
 
    else if (parsed_cmd->cmd_id == CMDID_VCPINFO) {
-      bool ok = true;
+      bool vcpinfo_ok = true;
+
+      // DDCA_MCCS_Version_Spec vcp_version_any = {0,0};
+
+      Feature_Set_Flags flags = 0;
+      if (parsed_cmd->flags & CMD_FLAG_RW_ONLY)
+         flags |= FSF_RW_ONLY;
+      if (parsed_cmd->flags & CMD_FLAG_RO_ONLY)
+         flags |= FSF_RO_ONLY;
+      if (parsed_cmd->flags & CMD_FLAG_WO_ONLY)
+         flags |= FSF_WO_ONLY;
 
-      DDCA_MCCS_Version_Spec vcp_version_any = {0,0};
       VCP_Feature_Set fset = create_feature_set_from_feature_set_ref(
-         // &feature_set_ref,
-         parsed_cmd->fref,
-         vcp_version_any,
-         false);       // force
+                                parsed_cmd->fref,
+                                parsed_cmd->mccs_vspec,
+                                flags);
       if (!fset) {
-         ok = false;
+         vcpinfo_ok = false;
       }
       else {
          if (parsed_cmd->output_level <= DDCA_OL_TERSE)
@@ -490,15 +519,10 @@ int main(int argc, char *argv[]) {
                report_vcp_feature_table_entry(pentry, 0);
             }
          }
+         free_vcp_feature_set(fset);
       }
 
-      if (ok) {
-         main_rc = EXIT_SUCCESS;
-      }
-      else {
-         // printf("Unrecognized VCP feature code or group: %s\n", val);
-         main_rc = EXIT_FAILURE;
-      }
+      main_rc = (vcpinfo_ok) ? EXIT_SUCCESS : EXIT_FAILURE;
    }
 
 #ifdef INCLUDE_TESTCASES
@@ -512,8 +536,8 @@ int main(int argc, char *argv[]) {
 
    else if (parsed_cmd->cmd_id == CMDID_DETECT) {
       ddc_ensure_displays_detected();
-      ddc_report_displays(DDC_REPORT_ALL_DISPLAYS, 0);
-
+      ddc_report_displays(/*include_invalid_displays=*/ true, 0);
+      main_rc = EXIT_SUCCESS;
    }
 
 #ifdef INCLUDE_TESTCASES
@@ -522,7 +546,7 @@ int main(int argc, char *argv[]) {
       bool ok = true;
       int ct = sscanf(parsed_cmd->args[0], "%d", &testnum);
       if (ct != 1) {
-         f0printf(FOUT, "Invalid test number: %s\n", parsed_cmd->args[0]);
+         f0printf(fout, "Invalid test number: %s\n", parsed_cmd->args[0]);
          ok = false;
       }
       else {
@@ -542,33 +566,33 @@ int main(int argc, char *argv[]) {
       char * fn = strdup( parsed_cmd->args[0] );
       // DBGMSG("Processing command loadvcp.  fn=%s", fn );
       Display_Handle * dh   = NULL;
-      bool ok = true;
+      bool loadvcp_ok = true;
       if (parsed_cmd->pdid) {
          Display_Ref * dref = get_display_ref_for_display_identifier(
                                  parsed_cmd->pdid, callopts | CALLOPT_ERR_MSG);
          if (!dref)
-            ok = false;
+            loadvcp_ok = false;
          else {
             ddc_open_display(dref, callopts | CALLOPT_ERR_MSG, &dh);  // rc == 0 iff dh, removed CALLOPT_ERR_ABORT
             if (!dh)
-               ok = false;
+               loadvcp_ok = false;
          }
       }
-      if (ok)
-         ok = loadvcp_by_file(fn, dh);
+      if (loadvcp_ok)
+         loadvcp_ok = loadvcp_by_file(fn, dh);
 
       // if we opened the display, we close it
       if (dh)
          ddc_close_display(dh);
       free(fn);
-      main_rc = (ok) ? EXIT_SUCCESS : EXIT_FAILURE;
+      main_rc = (loadvcp_ok) ? EXIT_SUCCESS : EXIT_FAILURE;
    }
 
    else if (parsed_cmd->cmd_id == CMDID_ENVIRONMENT) {
       dup2(1,2);   // redirect stderr to stdout
       ddc_ensure_displays_detected();   // *** NEEDED HERE ??? ***
 
-      f0printf(FOUT, "The following tests probe the runtime environment using multiple overlapping methods.\n");
+      f0printf(fout, "The following tests probe the runtime environment using multiple overlapping methods.\n");
       query_sysenv();
       main_rc = EXIT_SUCCESS;
    }
@@ -577,12 +601,12 @@ int main(int argc, char *argv[]) {
 #ifdef USE_USB
       dup2(1,2);   // redirect stderr to stdout
       ddc_ensure_displays_detected();   // *** NEEDED HERE ??? ***
-      f0printf(FOUT, "The following tests probe for USB connected monitors.\n");
+      f0printf(fout, "The following tests probe for USB connected monitors.\n");
       // DBGMSG("Exploring USB runtime environment...\n");
       query_usbenv();
       main_rc = EXIT_SUCCESS;
 #else
-      f0printf(FOUT, "ddcutil was not built with support for USB connected monitors\n");
+      f0printf(fout, "ddcutil was not built with support for USB connected monitors\n");
       main_rc = EXIT_FAILURE;
 #endif
    }
@@ -594,20 +618,21 @@ int main(int argc, char *argv[]) {
       main_rc = (is_monitor) ? EXIT_SUCCESS : EXIT_FAILURE;
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
+      main_rc = EXIT_FAILURE;
 #endif
    }
 
    else if (parsed_cmd->cmd_id == CMDID_INTERROGATE) {
       dup2(1,2);   // redirect stderr to stdout
-      // set_ferr(FOUT);    // ensure that all messages are collected - made unnecessary by dup2()
-      f0printf(FOUT, "Setting output level verbose...\n");
+      // set_ferr(fout);    // ensure that all messages are collected - made unnecessary by dup2()
+      f0printf(fout, "Setting output level verbose...\n");
       set_output_level(DDCA_OL_VERBOSE);
-      f0printf(FOUT, "Setting maximum retries...\n");
-      f0printf(FOUT, "Forcing --stats...\n");
+      f0printf(fout, "Setting maximum retries...\n");
+      f0printf(fout, "Forcing --stats...\n");
       parsed_cmd->stats_types = DDCA_STATS_ALL;
-      f0printf(FOUT, "Forcing --force-slave-address..\n");
+      f0printf(fout, "Forcing --force-slave-address..\n");
       i2c_force_slave_addr_flag = true;
-      f0printf(FOUT, "This command will take a while to run...\n\n");
+      f0printf(fout, "This command will take a while to run...\n\n");
       ddc_set_max_write_read_exchange_tries(MAX_MAX_TRIES);
       ddc_set_max_multi_part_read_tries(MAX_MAX_TRIES);
 
@@ -617,22 +642,24 @@ int main(int argc, char *argv[]) {
 #ifdef USE_USB
       // 7/2017: disable, USB attached monitors are rare, and this just
       // clutters the output
-      f0printf(FOUT, "\nSkipping USB environment exploration.\n");
-      f0printf(FOUT, "Issue command \"ddcutil usbenvironment --verbose\" if there are any USB attached monitors.\n");
+      f0printf(fout, "\nSkipping USB environment exploration.\n");
+      f0printf(fout, "Issue command \"ddcutil usbenvironment --verbose\" if there are any USB attached monitors.\n");
       // query_usbenv();
 #endif
-      f0printf(FOUT, "\nStatistics for environment exploration:\n");
+      f0printf(fout, "\nStatistics for environment exploration:\n");
       report_stats(DDCA_STATS_ALL);
       reset_stats();
 
-      f0printf(FOUT, "\n*** Detected Displays ***\n");
-      /* int display_ct =  */ ddc_report_displays(DDC_REPORT_ALL_DISPLAYS, 0 /* logical depth */);
+      f0printf(fout, "\n*** Detected Displays ***\n");
+      /* int display_ct =  */ ddc_report_displays(
+                                 true,   // include_invalid_displays
+                                 0);      // logical depth
       // printf("Detected: %d displays\n", display_ct);   // not needed
-      f0printf(FOUT, "\nStatistics for display detection:\n");
+      f0printf(fout, "\nStatistics for display detection:\n");
       report_stats(DDCA_STATS_ALL);
       reset_stats();
 
-      f0printf(FOUT, "Setting output level normal  Table features will be skipped...\n");
+      f0printf(fout, "Setting output level normal  Table features will be skipped...\n");
       set_output_level(DDCA_OL_NORMAL);
 
       GPtrArray * all_displays = ddc_get_all_displays();
@@ -640,17 +667,17 @@ int main(int argc, char *argv[]) {
          Display_Ref * dref = g_ptr_array_index(all_displays, ndx);
          assert( memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
          if (dref->dispno < 0) {
-            f0printf(FOUT, "\nSkipping invalid display on %s\n", dref_short_name_t(dref));
+            f0printf(fout, "\nSkipping invalid display on %s\n", dref_short_name_t(dref));
          }
          else {
-            f0printf(FOUT, "\nProbing display %d\n", dref->dispno);
+            f0printf(fout, "\nProbing display %d\n", dref->dispno);
             probe_display_by_dref(dref);
-            f0printf(FOUT, "\nStatistics for probe of display %d:\n", dref->dispno);
+            f0printf(fout, "\nStatistics for probe of display %d:\n", dref->dispno);
             report_stats(DDCA_STATS_ALL);
          }
          reset_stats();
       }
-      f0printf(FOUT, "\nDisplay scanning complete.\n");
+      f0printf(fout, "\nDisplay scanning complete.\n");
 
       main_rc = EXIT_SUCCESS;
    }
@@ -670,8 +697,7 @@ int main(int argc, char *argv[]) {
       // n. useful even if not much speed up, since avoids cluttering stats
       // with all the failures during detect
       Display_Ref * dref = NULL;
-      if (parsed_cmd->pdid->id_type == DISP_ID_BUSNO && parsed_cmd->nodetect) {
-  //  if (parsed_cmd->pdid->id_type == DISP_ID_BUSNO) {
+      if (parsed_cmd->pdid->id_type == DISP_ID_BUSNO && (parsed_cmd->flags & CMD_FLAG_NODETECT)) {
          int busno = parsed_cmd->pdid->busno;
          // is this really a monitor?
          I2C_Bus_Info * businfo = detect_single_bus(busno);
@@ -680,19 +706,21 @@ int main(int argc, char *argv[]) {
             dref->dispno = -1;     // should use some other value for unassigned vs invalid
             dref->pedid = businfo->edid;    // needed?
             // dref->pedid = i2c_get_parsed_edid_by_busno(parsed_cmd->pdid->busno);
-            dref->detail2 = businfo;
+            dref->detail = businfo;
             dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
             dref->flags |= DREF_DDC_IS_MONITOR;
             dref->flags |= DREF_TRANSIENT;
             if (!initial_checks_by_dref(dref)) {
-               f0printf(FOUT, "DDC communication failed for monitor on I2C bus /dev/i2c-%d\n", busno);
+               f0printf(fout, "DDC communication failed for monitor on I2C bus /dev/i2c-%d\n", busno);
                free_display_ref(dref);
                dref = NULL;
             }
+            // else
+               // check_dynamic_features(dref);    // the hook wrong location
             // DBGMSG("Synthetic Display_Ref");
          }
          else {
-            f0printf(FOUT, "No monitor detected on I2C bus /dev/i2c-%d\n", busno);
+            f0printf(fout, "No monitor detected on I2C bus /dev/i2c-%d\n", busno);
          }
       }
       else {
@@ -713,7 +741,7 @@ int main(int argc, char *argv[]) {
             {
                DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
                if (vspec.major < 2 && get_output_level() >= DDCA_OL_NORMAL) {
-                  f0printf(FOUT, "VCP (aka MCCS) version for display is undetected or less than 2.0. "
+                  f0printf(fout, "VCP (aka MCCS) version for display is undetected or less than 2.0. "
                         "Output may not be accurate.\n");
                }
             }
@@ -722,14 +750,18 @@ int main(int argc, char *argv[]) {
 
             case CMDID_CAPABILITIES:
                {
+                  // check_dynamic_features(dref);  // needed for this command?
+
                   Parsed_Capabilities * pcaps = perform_get_capabilities_by_display_handle(dh);
                   main_rc = (pcaps) ? EXIT_SUCCESS : EXIT_FAILURE;
-                  free(pcaps);
+                  free_parsed_capabilities(pcaps);
                   break;
                }
 
             case CMDID_GETVCP:
                {
+                  check_dynamic_features(dref);
+
                   Feature_Set_Flags flags = 0x00;
 
                   // DBGMSG("parsed_cmd->flags: 0x%04x", parsed_cmd->flags);
@@ -739,6 +771,12 @@ int main(int argc, char *argv[]) {
                      flags |= FSF_FORCE;
                   if (parsed_cmd->flags & CMD_FLAG_NOTABLE)
                      flags |= FSF_NOTABLE;
+                  if (parsed_cmd->flags & CMD_FLAG_RW_ONLY)
+                     flags |= FSF_RW_ONLY;
+                  if (parsed_cmd->flags & CMD_FLAG_RO_ONLY)
+                     flags |= FSF_RO_ONLY;
+                  if (parsed_cmd->flags & CMD_FLAG_WO_ONLY)
+                     flags |= FSF_WO_ONLY;
                   // char * s0 = feature_set_flag_names(flags);
                   // DBGMSG("flags: 0x%04x - %s", flags, s0);
                   // free(s0);
@@ -746,8 +784,6 @@ int main(int argc, char *argv[]) {
                   Public_Status_Code psc = app_show_feature_set_values_by_display_handle(
                         dh,
                         parsed_cmd->fref,
-                   //     parsed_cmd->show_unsupported,
-                   //     parsed_cmd->force,
                         flags
                         );
                   main_rc = (psc==0) ? EXIT_SUCCESS : EXIT_FAILURE;
@@ -756,7 +792,7 @@ int main(int argc, char *argv[]) {
 
             case CMDID_SETVCP:
                if (parsed_cmd->argct % 2 != 0) {
-                  f0printf(FOUT, "SETVCP command requires even number of arguments\n");
+                  f0printf(fout, "SETVCP command requires even number of arguments\n");
                   main_rc = EXIT_FAILURE;
                }
                else {
@@ -770,9 +806,7 @@ int main(int argc, char *argv[]) {
                              parsed_cmd->args[argNdx],
                              parsed_cmd->args[argNdx+1],
                              parsed_cmd->flags & CMD_FLAG_FORCE);
-                     // rc =  ERRINFO_STATUS(ddc_excp);
                      if (ddc_excp) {
-                        // errinfo_free(ddc_excp);
                         ERRINFO_FREE_WITH_REPORT(ddc_excp, report_freed_exceptions);
                         main_rc = EXIT_FAILURE;   // ???
                         break;
@@ -783,20 +817,20 @@ int main(int argc, char *argv[]) {
 
             case CMDID_SAVE_SETTINGS:
                if (parsed_cmd->argct != 0) {
-                  f0printf(FOUT, "SCS command takes no arguments\n");
+                  f0printf(fout, "SCS command takes no arguments\n");
                   main_rc = EXIT_FAILURE;
                }
                else if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
-                  f0printf(FOUT, "SCS command not supported for USB devices\n");
+                  f0printf(fout, "SCS command not supported for USB devices\n");
                   main_rc = EXIT_FAILURE;
                }
                else {
                   main_rc = EXIT_SUCCESS;
-                  Error_Info * ddc_excp = save_current_settings(dh);
+                  Error_Info * ddc_excp = ddc_save_current_settings(dh);
                   if (ddc_excp)  {
-                     f0printf(FOUT, "Save current settings failed. rc=%s\n", psc_desc(ddc_excp->status_code));
+                     f0printf(fout, "Save current settings failed. rc=%s\n", psc_desc(ddc_excp->status_code));
                      if (ddc_excp->status_code == DDCRC_RETRIES)
-                        f0printf(FOUT, "    Try errors: %s", errinfo_causes_string(ddc_excp) );
+                        f0printf(fout, "    Try errors: %s", errinfo_causes_string(ddc_excp) );
                      errinfo_report(ddc_excp, 0);   // ** ALTERNATIVE **/
                      errinfo_free(ddc_excp);
                      // ERRINFO_FREE_WITH_REPORT(ddc_excp, report_exceptions);
@@ -807,6 +841,8 @@ int main(int argc, char *argv[]) {
 
             case CMDID_DUMPVCP:
                {
+                  check_dynamic_features(dref);
+
                   Public_Status_Code psc =
                         dumpvcp_as_file(dh, (parsed_cmd->argct > 0)
                                                ? parsed_cmd->args[0]
@@ -819,20 +855,28 @@ int main(int argc, char *argv[]) {
                // DBGMSG("Case CMDID_READCHANGES");
                // report_parsed_cmd(parsed_cmd,0);
                app_read_changes_forever(dh);
+               main_rc = EXIT_SUCCESS;
                break;
 
             case CMDID_PROBE:
+               check_dynamic_features(dref);
+
                probe_display_by_dh(dh);
+               main_rc = EXIT_SUCCESS;
                break;
 
             default:
-              break;
+               main_rc = EXIT_FAILURE;
+               break;
             }
 
             ddc_close_display(dh);
          }
          if (dref->flags & DREF_TRANSIENT)
             free_display_ref(dref);
+      }   // if (dref)
+      else {
+         main_rc = EXIT_FAILURE;
       }
    }
 
@@ -840,7 +884,9 @@ int main(int argc, char *argv[]) {
       report_stats(parsed_cmd->stats_types);
       // report_timestamp_history();  // debugging function
    }
-   // DBGMSG("Done");
+   free_parsed_cmd(parsed_cmd);
 
+bye:
+   DBGMSF(main_debug, "Done.  main_rc=%d", main_rc);
    return main_rc;
 }
