@@ -107,7 +107,7 @@ Byte ddc_checksum(Byte * bytes, int len, bool altmode) {
    return checksum;
 }
 
-
+#ifdef TESTCASES
 void test_one_checksum(Byte * bytes, int len, bool altmode, Byte expected, char * spec_section) {
    unsigned char actual = ddc_checksum(bytes, len, altmode);
    char * hs =   hexstring(bytes, len);
@@ -132,6 +132,7 @@ void test_checksum() {
    test_one_checksum( (Byte[]) {0xf1, 0xf0, 0x82, 0xa1, 0x00}, 5, true,  0x83, "7.4");
    test_one_checksum( (Byte[]) {0x6f, 0xf0, 0x82, 0xa1, 0x00}, 5, true,  0x83, "7.4");
 }
+#endif
 
 
 bool valid_ddc_packet_checksum(Byte * readbuf) {
@@ -202,10 +203,10 @@ void dbgrpt_packet(DDC_Packet * packet, int depth) {
       switch(packet->type) {
       case (DDC_PACKET_TYPE_CAPABILITIES_RESPONSE):
       case (DDC_PACKET_TYPE_TABLE_READ_RESPONSE):
-         report_interpreted_multi_read_fragment(packet->parsed.multi_part_read_fragment, d0);
+         dbgrpt_interpreted_multi_read_fragment(packet->parsed.multi_part_read_fragment, d0);
          break;
       case (DDC_PACKET_TYPE_QUERY_VCP_RESPONSE):
-         report_interpreted_nontable_vcp_response(packet->parsed.nontable_response, d0);
+         dbgrpt_interpreted_nontable_vcp_response(packet->parsed.nontable_response, d0);
          break;
       default:
          // PROGRAM_LOGIC_ERROR("Unexpected packet type: -x%02x", packet->type);
@@ -246,10 +247,10 @@ void free_ddc_packet(DDC_Packet * packet) {
  *
  *  \param  max_size  size of buffer allocated for packet bytes
  *  \param  tag       debug string (may be NULL)
- *
  *  \return pointer to newly allocated #DDC_Packet
  */
-DDC_Packet * create_empty_ddc_packet(int max_size, const char * tag) {
+DDC_Packet *
+create_empty_ddc_packet(int max_size, const char * tag) {
    bool debug = false;
    DBGMSF(debug, "Starting. max_size=%d, tag=%s", max_size, (tag) ? tag : "(nil)");
 
@@ -281,13 +282,13 @@ DDC_Packet * create_empty_ddc_packet(int max_size, const char * tag) {
  *  \param  data_bytes   data bytes of packet
  *  \param  data_bytect  number of data bytes
  *  \param  tag          debug string (may be NULL)
- *
  *  \return  pointer to created packet
  */
-DDC_Packet * create_ddc_base_request_packet(
-                Byte *       data_bytes,
-                int          data_bytect,
-                const char * tag)
+DDC_Packet *
+create_ddc_base_request_packet(
+      Byte *       data_bytes,
+      int          data_bytect,
+      const char*  tag)
 {
    bool debug = false;
    DBGMSF(debug, "Starting.  bytes=%s, tag=%s", hexstring_t(data_bytes,data_bytect), tag);
@@ -314,20 +315,21 @@ DDC_Packet * create_ddc_base_request_packet(
 }
 
 
-/* Creates a DDC VCP table read request packet
+/** Creates a DDC VCP table read request packet
  *
- * Arguments:
- *    offset  offset value
- *    tag     debug string
- *
- * Returns:
- *    pointer to created capabilities request packet
+ *  \param  request_type     DDC_PACKET_TYPE_CAPABILITIES_REQUEST or
+ *                           DDC_PACKET_TYPE_TABLE_READ_REQUEST
+ *  \param  request_subtype  VCP code if reading table type feature, ignored for capabilities
+ *  \param  offset           offset value
+ *  \param  tag              debug string (may be null)
+ *  \return pointer to created capabilities request packet
  */
-DDC_Packet * create_ddc_multi_part_read_request_packet(
-                Byte request_type,
-                Byte request_subtype,
-                int offset,
-                const char * tag)
+DDC_Packet *
+create_ddc_multi_part_read_request_packet(
+      Byte         request_type,
+      Byte         request_subtype,
+      int          offset,
+      const char*  tag)
 {
    assert (request_type == DDC_PACKET_TYPE_CAPABILITIES_REQUEST ||
            request_type == DDC_PACKET_TYPE_TABLE_READ_REQUEST );
@@ -358,13 +360,16 @@ DDC_Packet * create_ddc_multi_part_read_request_packet(
 }
 
 
-/* Updates the offset in a multi part read request packet
+/** Updates the offset in a multi part read request packet
  *
- * Arguments:
- *    packet  address of packet
- *    offset  new offset value
+ *  \param  packet  address of packet
+ *  \offset offset  new offset value
  */
-void update_ddc_multi_part_read_request_packet_offset(DDC_Packet * packet, int new_offset) {
+void
+update_ddc_multi_part_read_request_packet_offset(
+      DDC_Packet * packet,
+      int          new_offset)
+{
    assert (packet->type == DDC_PACKET_TYPE_CAPABILITIES_REQUEST ||
            packet->type == DDC_PACKET_TYPE_TABLE_READ_REQUEST );
 
@@ -394,16 +399,18 @@ void update_ddc_multi_part_read_request_packet_offset(DDC_Packet * packet, int n
 }
 
 
-/* Creates a DDC VCP table write request packet
+/** Creates a DDC VCP table write request packet
  *
- * Arguments:
- *    offset  offset value
- *    tag     debug string
- *
- * Returns:
- *    pointer to created capabilities request packet
+ *  \param  request_type     always DDC_PACKET_TYPE_WRITE_REQUEST
+ *  \param  request_subtype  VCP code
+ *  \param  offset           offset value
+ *  \param  bytes_to_write   pointer to bytes to write
+ *  \param  bytect           number of bytes to write
+ *  \param  tag              debug string
+ *  \return pointer to newly created created multi-part-write request packet
  */
-DDC_Packet * create_ddc_multi_part_write_request_packet(
+DDC_Packet *
+create_ddc_multi_part_write_request_packet(
                 Byte   request_type,     // always DDC_PACKET_TYPE_WRITE_REQUEST
                 Byte   request_subtype,  // VCP code
                 int    offset,
@@ -433,14 +440,11 @@ DDC_Packet * create_ddc_multi_part_write_request_packet(
 }
 
 
-/* Creates a Get VCP request packet
+/** Creates a Get VCP request packet
  *
- * Arguments:
- *    vcp_code  VCP feature code
- *    tag       debug string
- *
- * Returns:
- *    pointer to created DDC packet
+ *  \param  vcp_code  VCP feature code
+ *  \param  tag       debug string
+ *  \return pointer to created DDC packet
  */
 DDC_Packet *
 create_ddc_getvcp_request_packet(Byte vcp_code, const char * tag)
@@ -455,15 +459,12 @@ create_ddc_getvcp_request_packet(Byte vcp_code, const char * tag)
 }
 
 
-/* Creates a Set VCP request packet
+/** Creates a Set VCP request packet
  *
- * Arguments:
- *    vcp_code  VCP feature code
- *    int       new value
- *    tag       debug string
- *
- * Returns:
- *    pointer to created DDC packet
+ *  \param   vcp_code  VCP feature code
+ *  \param   int       new value
+ *  \param   tag       debug string
+ *  \return  pointer to created DDC packet
  */
 DDC_Packet *
 create_ddc_setvcp_request_packet(Byte vcp_code, int new_value, const char * tag)
@@ -480,6 +481,11 @@ create_ddc_setvcp_request_packet(Byte vcp_code, int new_value, const char * tag)
 }
 
 
+/** Creates a request packet for Save Settings command.
+ *
+ *  \param  tag   debug string
+ *  \return pointer to created DDC packet
+ */
 DDC_Packet *
 create_ddc_save_settings_request_packet(const char * tag)
 {
@@ -506,10 +512,11 @@ create_ddc_save_settings_request_packet(const char * tag)
  *  \param packet_ptr_addr             where to return pointer to newly allocated #DDC_Packet
  *
  *  \retval 0
- *  \retval DDCRC_RESPONSE_ENVELOPE
- *  \retval DDCRC_DOUBLE_BYTE
- *  \retval DDCRC_PACKET_SIZE
- *  \retval DDCRC_CHECKSUM
+ *  \retval DDCRC_DDC_DATA
+ *  \retval DDCRC_RESPONSE_ENVELOPE   (deprecated)
+ *  \retval DDCRC_DOUBLE_BYTE         (deprecated)
+ *  \retval DDCRC_PACKET_SIZE         (deprecated)
+ *  \retval DDCRC_CHECKSUM            (deprecated)
  *
  *  The pointer returned at packet_ptr_addr is non-null iff the status code is 0.
  */
@@ -528,18 +535,18 @@ create_ddc_base_response_packet(
    DDC_Packet * packet = NULL;
    if (i2c_response_bytes[0] != 0x6e ) {
       DDCMSG(debug, "Unexpected source address 0x%02x, should be 0x6e", i2c_response_bytes[0]);
-      result = DDCRC_RESPONSE_ENVELOPE;
+      result = DDCRC_DDC_DATA;     // was DDCRC_RESPONSE_ENVELOPE
    }
    else {
       int data_ct = i2c_response_bytes[1] & 0x7f;
       // DBGMSG("data_ct=%d", data_ct);
       if (data_ct > MAX_DDC_DATA_SIZE) {
          if ( is_double_byte(&i2c_response_bytes[1])) {
-            result = DDCRC_DOUBLE_BYTE;
+            result = DDCRC_DDC_DATA;    // was DDCRC_DOUBLE_BYTE
             DDCMSG(debug, "Double byte in packet.");
          }
          else {
-            result = DDCRC_PACKET_SIZE;
+            result = DDCRC_DDC_DATA;     // was  DDCRC_PACKET_SIZE
             DDCMSG(debug,"Invalid data length in packet: %d exceeds MAX_DDC_DATA_SIZE", data_ct);
          }
       }
@@ -558,7 +565,7 @@ create_ddc_base_response_packet(
          if (calculated_checksum != actual_checksum) {
             DDCMSG(debug, "Actual checksum 0x%02x, expected 0x%02x",
                              actual_checksum, calculated_checksum);
-            result = DDCRC_CHECKSUM;
+            result = DDCRC_DDC_DATA;    //  was DDCRC_CHECKSUM
             free_ddc_packet(packet);
          }
       }
@@ -592,7 +599,8 @@ create_ddc_base_response_packet(
  *  \return 0 for success\n
  *          as from create_ddc_base_response_packet, indicating malformed response
  *  \retval DDCRC_NULL_RESPONSE
- *  \retval DDCRC_RESPONSE_TYPE
+ *  \retval DDCRC_RESPONSE_TYPE (deprecated)
+ *  \retval DDCRC_DDC_DATA
  *
  *  The pointer returned at packet_ptr_addr is non-null iff the status code is 0.
  */
@@ -619,7 +627,7 @@ create_ddc_response_packet(
          result = DDCRC_NULL_RESPONSE;
       }
       else if ( get_data_start(*packet_ptr_addr)[0] != expected_type) {
-         result = DDCRC_RESPONSE_TYPE;
+         result = DDCRC_DDC_DATA;      // was: DDCRC_RESPONSE_TYPE
       }
    }
 
@@ -649,6 +657,8 @@ create_ddc_response_packet(
 
 // Capabilities response data 
 
+#ifdef OLD
+// deprecated, use more general dbgrpt_interpreted_multi_read_fragment()
 void
 report_interpreted_capabilities(
       Interpreted_Multi_Part_Read_Fragment * interpreted,
@@ -660,23 +670,26 @@ report_interpreted_capabilities(
    rpt_vstring(d1,    "fragment length: %d", interpreted->fragment_length);
    rpt_vstring(d1,    "text:            |%.*s|", interpreted->fragment_length, interpreted->bytes);
 }
+#endif
 
 
 /** Interprets the bytes of a multi part read response.
  *
- *  \param  response_type
- *  \param  data_bytes
- *  \param  bytect
- *  \param  aux_data       pointer to #Interpreted_Multi_Part_Read_Fragment to fill in
- *  \retval 0 success
- *  \retval  DDCRC_INVALID_DATA
+ *  \param   response_type
+ *  \param   data_bytes
+ *  \param   bytect
+ *  \param   aux_data       pointer to #Interpreted_Multi_Part_Read_Fragment to fill in
+ *  \retval  0 success
+ *  \retval  DDCRC_INVALID_DATA (deprecated)
+ *  \retval  DDCRC_DDC_DATA
  */
 Status_DDC
 interpret_multi_part_read_response(
        DDC_Packet_Type  response_type,
-       Byte * data_bytes,
-       int    bytect,
-       Interpreted_Multi_Part_Read_Fragment * aux_data)  //   Interpreted_Capabilities_Fragment * aux_data,
+       Byte*            data_bytes,
+       int              bytect,
+       Interpreted_Multi_Part_Read_Fragment*
+                        aux_data)  //   Interpreted_Capabilities_Fragment * aux_data,
 {
    bool debug = false;
    int result = DDCRC_OK;
@@ -685,7 +698,7 @@ interpret_multi_part_read_response(
    if (bytect < 3 || bytect > 35) {
       // if (debug)
       DDCMSG(debug, "Invalid response data length: %d", bytect);
-      result = COUNT_STATUS_CODE(DDCRC_INVALID_DATA);
+      result = COUNT_STATUS_CODE(DDCRC_DDC_DATA);   // was DDCRC_INVALID_DATA
    }
    else {
       assert( data_bytes[0] == response_type);             // table read reply opcode    // CHANGED
@@ -709,7 +722,7 @@ interpret_multi_part_read_response(
 
 
 void
-report_interpreted_multi_read_fragment(
+dbgrpt_interpreted_multi_read_fragment(
       Interpreted_Multi_Part_Read_Fragment * interpreted,
       int depth)
 {
@@ -789,8 +802,7 @@ interpret_vcp_feature_response_std(
    if (bytect != 8) {
       DDCMSG(debug, "Invalid response data length: %d, should be 8, response data bytes: %s",
                        bytect, hexstring3_t(vcp_data_bytes, bytect, " ", 4, false));
-      COUNT_STATUS_CODE(DDCRC_INVALID_DATA);
-      result = DDCRC_INVALID_DATA;
+      result = COUNT_STATUS_CODE(DDCRC_DDC_DATA);   // was DDCRC_INVALID_DATA
    }
    else {
       // overlay Vcp_Response on the data bytes of the response
@@ -804,7 +816,7 @@ interpret_vcp_feature_response_std(
          DDCMSG(debug, "Unexpected VCP opcode 0x%02x, should be 0x%02x, response data bytes: %s",
                           vcpresp->vcp_opcode, requested_vcp_code,
                           hexstring3_t(vcp_data_bytes, bytect, " ", 4, false));
-         result = COUNT_STATUS_CODE(DDCRC_INVALID_DATA);
+         result = COUNT_STATUS_CODE(DDCRC_DDC_DATA);   // was DDCRC_INVALID_DATA
       }
 
       else if (vcpresp->result_code != 0) {
@@ -821,7 +833,7 @@ interpret_vcp_feature_response_std(
             DDCMSG(debug, "Unexpected result code: 0x%02x, response_data_bytes: %s",
                              vcpresp->result_code,
                              hexstring3_t(vcp_data_bytes, bytect, " ", 4, false));
-            result = COUNT_STATUS_CODE(DDCRC_INVALID_DATA);
+            result = COUNT_STATUS_CODE(DDCRC_DDC_DATA);   // was DDCRC_INVALID_DATA
          }
       }
 
@@ -851,8 +863,8 @@ interpret_vcp_feature_response_std(
 }
 
 
-
-void report_interpreted_nontable_vcp_response(
+void
+dbgrpt_interpreted_nontable_vcp_response(
         Parsed_Nontable_Vcp_Response * interpreted, int depth)
 {
    rpt_vstring(depth,"VCP code:         0x%02x", interpreted->vcp_code);
@@ -867,13 +879,15 @@ void report_interpreted_nontable_vcp_response(
 }
 
 
-void   report_parsed_vcp_response(Parsed_Vcp_Response * response, int depth) {
+void
+dbgrpt_parsed_vcp_response(Parsed_Vcp_Response * response, int depth)
+{
    rpt_vstring(depth, "Parsed_Vcp_Reponse at %p:", response);
 
    rpt_vstring(depth, "response_type:   %d",   response->response_type);
    if (response->response_type == DDCA_NON_TABLE_VCP_VALUE) {
       rpt_vstring(depth, "non_table_response at %p:", response->non_table_response);
-      report_interpreted_nontable_vcp_response(response->non_table_response, depth+1);
+      dbgrpt_interpreted_nontable_vcp_response(response->non_table_response, depth+1);
    }
    else {
       rpt_vstring(depth, "table_response at %p", response->table_response);
@@ -881,13 +895,11 @@ void   report_parsed_vcp_response(Parsed_Vcp_Response * response, int depth) {
 }
 
 
-
-
 //
 // Response packets 
 //
 
-/** Creates a #DDC_Packet for a DDC response.
+/** Creates a #DDC_Packet from a raw DDC response.
  *
  *  \param i2c_response_bytes          pointer to raw packet bytes
  *  \param response_bytes_buffer_size  size of buffer pointed to by **i2c_response_bytes**
@@ -905,7 +917,8 @@ void   report_parsed_vcp_response(Parsed_Vcp_Response * response, int depth) {
  *  The contents of **expected_subtype** depends on the value of **expected_type**.
  *  For DDC_PACKET_TYPE_QUERY_VCP_RESPONSE it is the VCP feature code.
  */
-Status_DDC create_ddc_typed_response_packet(
+Status_DDC
+create_ddc_typed_response_packet(
       Byte*           i2c_response_bytes,
       int             response_bytes_buffer_size,
       DDC_Packet_Type expected_type,
@@ -956,7 +969,7 @@ Status_DDC create_ddc_typed_response_packet(
          break;
 
       default:
-         rc = DDCL_INTERNAL_ERROR;
+         rc = DDCRC_INTERNAL_ERROR;
          DBGMSG("Unhandled case. expected_type=%d", expected_type);
          break;
       }
@@ -988,10 +1001,10 @@ create_ddc_multi_part_read_response_packet(
 
    DDC_Packet * packet = NULL;
    Status_DDC rc = create_ddc_response_packet(i2c_response_bytes,
-                                                     response_bytes_buffer_size,
-                                                     DDC_PACKET_TYPE_TABLE_READ_RESPONSE,
-                                                     tag,
-                                                     &packet);
+                                              response_bytes_buffer_size,
+                                              DDC_PACKET_TYPE_TABLE_READ_RESPONSE,
+                                              tag,
+                                              &packet);
    if (rc != 0) {
       // DBGMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_description(rc), packet);
       TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc(rc), packet);
@@ -1005,7 +1018,7 @@ create_ddc_multi_part_read_response_packet(
          DDCMSG(debug, "Invalid data fragment_length_wo_null: %d", data_len);
          if (IS_REPORTING_DDC())
             dbgrpt_packet(packet, 1);
-         rc = COUNT_STATUS_CODE(DDCRC_INVALID_DATA);
+         rc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);    // was DDCRC_INVALID_DATA
       }
       else {
          Interpreted_Multi_Part_Read_Fragment * aux_data = calloc(1, sizeof(Interpreted_Multi_Part_Read_Fragment));
@@ -1060,7 +1073,7 @@ create_ddc_getvcp_response_packet(
          DDCMSG(debug, "Invalid data length: %d, should be 8", data_len);
          if ( IS_REPORTING_DDC() )
             dbgrpt_packet(packet, 1);
-         rc = COUNT_STATUS_CODE(DDCRC_INVALID_DATA);
+         rc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);     // was DDCRC_INVALID_DATA
       }
       else {
          Parsed_Nontable_Vcp_Response * aux_data = calloc(1, sizeof(Parsed_Nontable_Vcp_Response));
@@ -1093,12 +1106,13 @@ create_ddc_getvcp_response_packet(
  *
  *  This is the aux_data field of #DDC_Packet
  *
- * \param   packet  pointer to digested packet (not raw bytes)
+ * \param   packet      pointer to digested packet (not raw bytes)
  * \param   make_copy  if true, make a copy of the aux_data field,\n
  *                     if false, just return a pointer to it
- * \param   interpreted_ptr  where to return newly allocated #Parsed_Nontable_Vcp_Response
+ * \param   interpreted_loc  where to return pointer to newly allocated #Parsed_Nontable_Vcp_Response
  * \retval  0    success
- * \retval  DDCRC_RESPONSE_TYPE  not a VCP response packet
+ * \retval  DDCRC_RESPONSE_TYPE  not a VCP response packet  (deprecated)
+ * \retval  DDCRC_DDC_DATA       not a VCP response packet
  *
  * The value pointed to by **interpreted_ptr** is non-null iff the returned status code is 0.
  */
@@ -1106,30 +1120,28 @@ Status_DDC
 get_interpreted_vcp_code(
        DDC_Packet *            packet,
        bool                    make_copy,
-       Parsed_Nontable_Vcp_Response ** interpreted_ptr)
+       Parsed_Nontable_Vcp_Response ** interpreted_loc)
 {
    bool debug = false;
    DBGMSF(debug, "Starting");
 
    Status_DDC rc = DDCRC_OK;
    if (packet->type != DDC_PACKET_TYPE_QUERY_VCP_RESPONSE) {
-      COUNT_STATUS_CODE(DDCRC_RESPONSE_TYPE);
-      rc = DDCRC_RESPONSE_TYPE;
-      *interpreted_ptr = NULL;
+      rc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);      // was DDCRC_RESPONSE_TYPE
+      *interpreted_loc = NULL;
    }
    else {
       if (make_copy) {
-         Parsed_Nontable_Vcp_Response * copy =
-            malloc(sizeof(Parsed_Nontable_Vcp_Response));
+         Parsed_Nontable_Vcp_Response * copy = malloc(sizeof(Parsed_Nontable_Vcp_Response));
          memcpy(copy, packet->parsed.nontable_response, sizeof(Parsed_Nontable_Vcp_Response));
-         *interpreted_ptr = copy;
+         *interpreted_loc = copy;
       }
       else {
-         *interpreted_ptr = packet->parsed.nontable_response;
+         *interpreted_loc = packet->parsed.nontable_response;
       }
    }
    DBGMSF(debug, "Returning %d: %s\n", rc, psc_desc(rc) );
-   assert( (rc == 0 && *interpreted_ptr) || (rc && !*interpreted_ptr));
+   assert( (rc == 0 && *interpreted_loc) || (rc && !*interpreted_loc));
    return rc;
 }
 
