@@ -403,7 +403,9 @@ void show_specific_status_counts(Status_Code_Counts * pcounts) {
    // in all but the most recent distros, .e.g SUSE 13.2 is v 2.18
    // gpointer * keysp = g_hash_table_get_keys_as_array(pcounts->error_counts_hash, &keyct);
    GList * glist = g_hash_table_get_keys(pcounts->error_counts_hash);
+   // copies the pointers in the linked list to the array, does not duplicate
    gpointer * keysp = g_list_to_g_array(glist, &keyct);
+   g_list_free(glist);
    if (debug) {
       DBGMSG("Keys.  keyct=%d", keyct);
       for (int ndx = 0; ndx < keyct; ndx++) {
@@ -481,6 +483,7 @@ int get_true_io_error_count(Status_Code_Counts * pcounts) {
      // gpointer * keysp = g_hash_table_get_keys_as_array(pcounts->error_counts_hash, &keyct);
      GList * glist = g_hash_table_get_keys(pcounts->error_counts_hash);
      gpointer * keysp = g_list_to_g_array(glist, &keyct);
+     g_list_free(glist);
 
      int summed_ct = 0;
 
@@ -596,7 +599,7 @@ char * sleep_strategy_desc(int sleep_strategy) {
  *  2. Records the sleep event.
  *  3. Sleeps for period determined.
  *
- * @param io_mode     communication mechanism (must be #DDCA_IO_DEVI2C)
+ * @param io_mode     communication mechanism (must be #DDCA_IO_I2C)
  * @param event_type  reason for sleep (currently only #SE_DDC_NULL - DDC Null Response)
  * @param occno       occurrence count of event
  *
@@ -613,7 +616,7 @@ call_dynamic_tuned_sleep(
    bool debug =  false || debug_sleep_stats_mutex;
 
    int sleep_time_millis = 0;
-   assert(io_mode == DDCA_IO_DEVI2C);
+   assert(io_mode == DDCA_IO_I2C);
    assert(event_type == SE_DDC_NULL);
 
    switch(event_type) {
@@ -651,7 +654,7 @@ call_dynamic_tuned_sleep_i2c(
       Sleep_Event_Type event_type,
       int occno)
 {
-   call_dynamic_tuned_sleep(DDCA_IO_DEVI2C, event_type, occno);
+   call_dynamic_tuned_sleep(DDCA_IO_I2C, event_type, occno);
 }
 
 
@@ -682,7 +685,7 @@ void call_tuned_sleep(DDCA_IO_Mode io_mode, Sleep_Event_Type event_type) {
 
    assert(event_type != SE_DDC_NULL);
 
-   case DDCA_IO_DEVI2C:
+   case DDCA_IO_I2C:
       switch(event_type) {
       case (SE_WRITE_TO_READ):
             sleep_time_millis = DDC_TIMEOUT_MILLIS_DEFAULT;
@@ -775,7 +778,7 @@ void call_tuned_sleep(DDCA_IO_Mode io_mode, Sleep_Event_Type event_type) {
  *  @param event_type sleep event type
  */
 void call_tuned_sleep_i2c(Sleep_Event_Type event_type) {
-   call_tuned_sleep(DDCA_IO_DEVI2C, event_type);
+   call_tuned_sleep(DDCA_IO_I2C, event_type);
 }
 
 
