@@ -169,23 +169,77 @@ typedef struct feature_subset_table_entry_s {
 } Feature_Subset_Table_Entry;
 
 const Feature_Subset_Table_Entry subset_table[] = {
-   {VCP_SUBSET_ALL,       CMDID_GETVCP|CMDID_VCPINFO, 3, "ALL",       "All known features"},
-   {VCP_SUBSET_SUPPORTED, CMDID_GETVCP,               3, "SUPPORTED", "All known features that are valid for the display"},
+   // special handling
+   {VCP_SUBSET_KNOWN,     CMDID_GETVCP|CMDID_VCPINFO, 3, "KNOWN",     "All features known to ddcutil that are valid for the display"},
+   {VCP_SUBSET_KNOWN,     CMDID_GETVCP|CMDID_VCPINFO, 3, "ALL",       "Same as KNOWN"},
+// {VCP_SUBSET_SUPPORTED, CMDID_GETVCP,               3, "SUPPORTED", "All known features that are valid for the display"},
    {VCP_SUBSET_SCAN,      CMDID_GETVCP,               3, "SCAN",      "All feature codes 00..FF, except those known to be WO"},
-   {VCP_SUBSET_KNOWN,     CMDID_GETVCP|CMDID_VCPINFO, 3, "KNOWN",     "All features known by ddcutil"},
+   {VCP_SUBSET_MFG,       CMDID_GETVCP,               3, "MANUFACTURER", "Manufacturer specific codes"},
+   {VCP_SUBSET_MFG,       CMDID_GETVCP,               3, "MFG",        "Same as MANUFACTURER"},
+
+   // ddcutil defined groups
    {VCP_SUBSET_PROFILE,   CMDID_GETVCP|CMDID_VCPINFO, 3, "PROFILE",   "Features for color profile management"},
    {VCP_SUBSET_COLOR,     CMDID_GETVCP|CMDID_VCPINFO, 3, "COLOR",     "Color related features"},
    {VCP_SUBSET_LUT,       CMDID_GETVCP|CMDID_VCPINFO, 3, "LUT",       "LUT related features"},
+
+   // by MCCS spec group
+   {VCP_SUBSET_CRT,       CMDID_GETVCP|CMDID_VCPINFO, 3, "CRT",       "CRT related features"},
    {VCP_SUBSET_AUDIO,     CMDID_GETVCP|CMDID_VCPINFO, 3, "AUDIO",     "Audio related features"},
    {VCP_SUBSET_WINDOW,    CMDID_GETVCP|CMDID_VCPINFO, 3, "WINDOW",    "Window related features"},
    {VCP_SUBSET_TV,        CMDID_GETVCP|CMDID_VCPINFO, 2, "TV",        "TV related features"},
+   {VCP_SUBSET_DPVL,      CMDID_GETVCP|CMDID_VCPINFO, 3, "DPVL",      "DPVL related features"},
    {VCP_SUBSET_PRESET,                 CMDID_VCPINFO, 3, "PRESET",    "Presets"},     // all WO
-   {VCP_SUBSET_MFG,       CMDID_GETVCP,               3, "MANUFACTURER", "Manufacturer specific codes"},
+
+   // by feature type
    {VCP_SUBSET_TABLE,     CMDID_GETVCP|CMDID_VCPINFO, 3, "TABLE",     "Table type features"},
+   {VCP_SUBSET_SCONT,     CMDID_GETVCP|CMDID_VCPINFO, 3, "SCONT",     "Simple Continuous features"},
+   {VCP_SUBSET_CCONT,     CMDID_GETVCP|CMDID_VCPINFO, 3, "CCONT",     "Complex Continuous features"},
+   {VCP_SUBSET_CONT,      CMDID_GETVCP|CMDID_VCPINFO, 3, "CONT",      "All Continuous features"},
+   {VCP_SUBSET_SNC,       CMDID_GETVCP|CMDID_VCPINFO, 3, "SNC",       "Simple NC features"},
+   {VCP_SUBSET_CNC,       CMDID_GETVCP|CMDID_VCPINFO, 3, "CNC",       "Complex NC features"},
+   {VCP_SUBSET_NC_WO,                  CMDID_VCPINFO, 4, "NC_WO",     "Write-only NC features"},
+   {VCP_SUBSET_NC_CONT,   CMDID_GETVCP|CMDID_VCPINFO, 4, "NC_CONT",   "NC features with continuous subrange"},
+   {VCP_SUBSET_NC,        CMDID_GETVCP|CMDID_VCPINFO, 2, "NC",        "All NC features"},
 };
 const int subset_table_ct = sizeof(subset_table)/sizeof(Feature_Subset_Table_Entry);
 
 
+char * assemble_command_argument_help() {
+   // quick and dirty check that tables are in sync
+   // +2 for VCP_SUBSET_SINGLE_FEATURE, VCP_SUBSET_NONE
+   // -1 for double VCP_SUBSET_KNOWN
+   // -1 for double VCP_SUBSET_MFG
+   assert(subset_table_ct+(2-1-1) == vcp_subset_count);
+
+   GString * buf = g_string_sized_new(1000);
+   g_string_append(buf,
+         "Command Arguments\n"
+         "  getvcp, vcpinfo:\n"
+         "    <feature-code-or-group> can be any of the following:\n"
+         "      - the hex feature code for a specific feature, with or without a leading 0x,\n"
+         "        e.g. 10 or 0x10\n");
+
+   for (int ndx = 0; ndx < subset_table_ct; ndx++) {
+      g_string_append_printf(buf, "      - %-10s - %s\n",  subset_table[ndx].subset_name, subset_table[ndx].subset_desc);
+   }
+
+   g_string_append(buf,
+   "    Keywords can be abbreviated to the first 3 characters.\n"
+   "    Case is ignored.  e.g. \"COL\", \"pro\"\n"
+   "\n"
+   "  setvcp:\n"
+   "    <feature-code>: hexadecimal feature code, with or without a leading 0x,\n"
+   "       e.g. 10 or 0x10\n"
+   "    [+|-] optionally indicate a relative value change, must be surrounded by blanks\n"
+   "    <new-value>: a decimal number in the range 0..255, or a single byte hex value,\n"
+   "       e.g. 0x80\n");
+
+   char * result = buf->str;
+   g_string_free(buf, false);
+   // DBGMSG("Returning: |%s|", result);
+   return result;
+}
+
 
 VCP_Feature_Subset find_subset(char * name, int cmd_id) {
    assert(name && (cmd_id == CMDID_GETVCP || cmd_id == CMDID_VCPINFO));
@@ -263,58 +317,70 @@ bool validate_output_level(Parsed_Cmd* parsed_cmd) {
 
 char * commands_list_help =
        "Commands:\n"
-       "   detect                               Detect monitors\n"
-       "   capabilities                         Query monitor capabilities string\n"
+       "   detect                                  Detect monitors\n"
+       "   capabilities                            Query monitor capabilities string\n"
 //     "   info\n"
 //     "   listvcp\n"
-       "   vcpinfo (feature-code-or-group)      Show VCP feature characteristics\n"
-       "   getvcp <feature-code-or-group>       Get VCP feature value(s)\n"
-       "   setvcp <feature-code> <new-value>    Set VCP feature value\n"
-       "   dumpvcp (filename)                   Write profile related settings to file\n"
-       "   loadvcp <filename>                   Load profile related settings from file\n"
+       "   vcpinfo (feature-code-or-group)         Show VCP feature characteristics\n"
+       "   getvcp <feature-code-or-group>          Report VCP feature value(s)\n"
+       "   setvcp <feature-code> [+|-] <new-value> Set VCP feature value\n"
+       "   dumpvcp (filename)                      Write color profile related settings to file\n"
+       "   loadvcp <filename>                      Load profile related settings from file\n"
 #ifdef INCLUDE_TESTCASES
        "   testcase <testcase-number>\n"
        "   listtests\n"
 #endif
-       "   environment                          Probe execution environment\n"
+       "   environment                             Probe execution environment\n"
 #ifdef USE_USB
-       "   usbenv                               Probe for USB connected monitors\n"
+       "   usbenv                                  Probe for USB connected monitors\n"
 #endif
-       "   probe                                Probe monitor abilities\n"
-       "   interrogate                          Report everything possible\n"
+       "   probe                                   Probe monitor abilities\n"
+       "   interrogate                             Report everything possible\n"
 #ifdef USE_USB
-       "   chkusbmon                            Check if USB device is monitor (for UDEV)\n"
+       "   chkusbmon                               Check if USB device is monitor (for UDEV)\n"
 #endif
-       "   watch                                Watch display for reported changes (under development)\n"
+       "   watch                                   Watch display for reported changes (under development)\n"
        "\n";
 
+#ifdef OLD
 char * command_argument_help =
        "Command Arguments\n"
        "  getvcp, vcpinfo:\n"
        "    <feature-code-or-group> can be any of the following:\n"
        "      - the hex feature code for a specific feature, with or without a leading 0x,\n"
        "        e.g. 10 or 0x10\n"
-       "      - KNOWN     - all feature codes known to ddcutil\n"
-       "      - ALL       - like KNOWN, but implies --show-unsupported\n"
-       "      - SCAN      - scan all feature codes 0x00..0xff\n"
-       "      - COLOR     - all color related feature codes\n"
-       "      - PROFILE   - color related codes for profile management\n"
-       "      - LUT       - LUT related features\n"
-       "      - AUDIO     - audio features\n"
-       "      - WINDOW    - window operations (e.g. PIP)\n"
-       "      - TV        - TV related settings\n"
-       "      - PRESET    - MCCS codes classed as PRESET\n"
+       "      - KNOWN        - all feature codes known to ddcutil\n"
+       "      - ALL          - like KNOWN, but implies --show-unsupported\n"
+       "      - SCAN         - scan all feature codes 0x00..0xff\n"
+       "      - COLOR        - all color related feature codes\n"
+       "      - PROFILE      - color related codes for profile management\n"
+       "      - LUT          - LUT related features\n"
+       "      - AUDIO        - audio features\n"
+       "      - WINDOW       - window operations (e.g. PIP)\n"
+       "      - TV           - TV related settings\n"
+       "      - PRESET       - MCCS codes classed as PRESET\n"
        "      - MANUFACTURER - manufacturer specific codes\n"
-       "      - TABLE     - table type features\n"
+       "      - MFG          - same as MANUFACTURER\n"
+       "      - TABLE        - Table type features\n"
+       "      - SCONT        - simple Continuous features\n"
+       "      - CCONT        - complex Continuous features\n"
+       "      - CONT         - all Continuous features\n"
+       "      - SNC          - simple Non-Continuous features\n"
+       "      - CNC          - complex Non-Continuous features, using multiple bytes\n"
+       "      - NC_WO        - write/only Non-Continuous features\n"
+       "      - NC_CONT      - features classed as NC having a continuous subrange\n"
+       "      - NC           - all Non-Continuous features\n"
        "    Keywords can be abbreviated to the first 3 characters.\n"
        "    Case is ignored.  e.g. \"COL\", \"pro\"\n"
        "\n"
        "  setvcp:\n"
        "    <feature-code>: hexadecimal feature code, with or without a leading 0x,\n"
        "       e.g. 10 or 0x10\n"
+       "    [+|-] optionally indicate a relative value change, must be surrounded by blanks\n"
        "    <new-value>: a decimal number in the range 0..255, or a single byte hex value,\n"
        "       e.g. 0x80\n"
       ;
+#endif
 
 char * monitor_selection_option_help =
        "Monitor Selection:\n"
