@@ -3,7 +3,7 @@
  * Parse the command line using the glib goption functions.
  *
  * <copyright>
- * Copyright (C) 2014-2016 Sanford Rockowitz <rockowitz@minsoft.com>
+ * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -93,10 +93,6 @@ gboolean output_arg_func(const gchar* option_name,
       output_level = DDCA_OL_VERBOSE;
    else if (streq(option_name, "-t")  || streq(option_name, "--terse") || streq(option_name, "--brief") )
       output_level = DDCA_OL_TERSE;
-#ifdef OLD
-   else if (streq(option_name, "-p") || streq(option_name, "--program"))
-      output_level = OL_PROGRAM;
-#endif
    else {
       PROGRAM_LOGIC_ERROR("Unexpected option_name: %s", option_name);
       g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED,
@@ -189,13 +185,15 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    gboolean nodetect_flag  = false;
    gboolean async_flag     = false;
    gboolean report_freed_excp_flag = false;
-   gboolean notable_flag   = false;
-// gboolean myhelp_flag    = false;
-// gboolean myusage_flag   = false;
+   gboolean notable_flag   = true;
+   gboolean rw_only_flag   = false;
+   gboolean ro_only_flag   = false;
+   gboolean wo_only_flag   = false;
    char *   mfg_id_work    = NULL;
    char *   modelwork      = NULL;
    char *   snwork         = NULL;
    char *   edidwork       = NULL;
+   char *   mccswork       = NULL;   // MCCS version
 // char *   tracework      = NULL;
    char**   cmd_and_args   = NULL;
    gchar**  trace_classes  = NULL;
@@ -225,7 +223,6 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       {"edid",    'e',  0, G_OPTION_ARG_STRING,   &edidwork,         "Monitor EDID",            "256 char hex string" },
 
       // output control
-
       {"ddc",     '\0', 0, G_OPTION_ARG_NONE,     &ddc_flag,         "Report DDC protocol and data errors", NULL},
       {"verbose", 'v',  G_OPTION_FLAG_NO_ARG,
                            G_OPTION_ARG_CALLBACK, output_arg_func,   "Show extended detail",             NULL},
@@ -235,7 +232,14 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
                            G_OPTION_ARG_CALLBACK, output_arg_func,   "Show brief detail",                NULL},
       {"show-unsupported",
                   'U',  0, G_OPTION_ARG_NONE,     &show_unsupported_flag, "Report unsupported features", NULL},
-      {"notable", '\0', 0, G_OPTION_ARG_NONE,     &notable_flag,     "Ignore table type feature codes",  NULL},
+      {"notable", '\0', G_OPTION_FLAG_HIDDEN,
+                           G_OPTION_ARG_NONE,     &notable_flag,     "Exclude table type feature codes",  NULL},
+      {"no-table",'\0', 0, G_OPTION_ARG_NONE,     &notable_flag,     "Exclude table type feature codes",  NULL},
+      {"show-table",'\0',G_OPTION_FLAG_REVERSE,
+                           G_OPTION_ARG_NONE,     &notable_flag,     "Report table type feature codes",  NULL},
+      {"rw",      '\0', 0, G_OPTION_ARG_NONE,     &rw_only_flag,     "Include only RW features",         NULL},
+      {"ro",      '\0', 0, G_OPTION_ARG_NONE,     &ro_only_flag,     "Include only RO features",         NULL},
+      {"wo",      '\0', 0, G_OPTION_ARG_NONE,     &wo_only_flag,     "Include only WO features",         NULL},
 
       // tuning
       {"maxtries",'\0', 0, G_OPTION_ARG_STRING,   &maxtrywork,       "Max try adjustment",  "comma separated list" },
@@ -269,6 +273,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 
       // other
       {"version", 'V',  0, G_OPTION_ARG_NONE,     &version_flag,     "Show version information", NULL},
+      {"mccs",    '\0', 0, G_OPTION_ARG_STRING,   &mccswork,         "MCCS version",            "major.minor" },
 
       {G_OPTION_REMAINING,
                  '\0', 0,  G_OPTION_ARG_STRING_ARRAY, &cmd_and_args, "ARGUMENTS description",   "command [arguments...]"},
@@ -289,8 +294,14 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    // const char * pieces2[] = {command_argument_help, "  Recognized trace classes: ", trace_group_string, "\n\n"};
    // command_argument_help = strjoin(pieces, 4, NULL);
 
-   const char * pieces3[] = {commands_list_help, command_argument_help};
+   // const char * pieces3[] = {commands_list_help, command_argument_help};
+   // char * help_summary = strjoin(pieces3, 2, NULL);
+
+   char * cmd_args_help = assemble_command_argument_help();
+   // const char * pieces3[] = {commands_list_help, command_argument_help, cmd_args_help}; // TEMP commands_list_help
+   const char * pieces3[] = {commands_list_help, cmd_args_help};
    char * help_summary = strjoin(pieces3, 2, NULL);
+   free(cmd_args_help);
 
    const char * pieces4[] = {monitor_selection_option_help,
                              tracing_multiple_call_option_help,
@@ -306,9 +317,11 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 
    // on --help, comes after usage line, before option detail
    g_option_context_set_summary(context, help_summary);
+   free(help_summary);
 
    // on --help, comes at end after option detail
    g_option_context_set_description(context, help_description);
+   free(help_description);
 
    g_option_context_set_help_enabled(context, true);
    // bool ok = false;
@@ -326,35 +339,45 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 
    int explicit_display_spec_ct = 0;  // number of ways the display is explicitly specified
 
-   parsed_cmd->ddcdata          = ddc_flag;
-   // parsed_cmd->force            = force_flag;
-   parsed_cmd->force_slave_addr = force_slave_flag;
-   // parsed_cmd->show_unsupported = show_unsupported_flag;
+   int rwo_flag_ct = 0;
+   if (rw_only_flag)   rwo_flag_ct++;
+   if (ro_only_flag)   rwo_flag_ct++;
+   if (wo_only_flag)   rwo_flag_ct++;
+   if (rwo_flag_ct > 1) {
+      fprintf(stderr, "Options -rw-only, --ro-only, --wo-only are mutually exclusive\n");
+      ok = false;
+   }
+
+
+#define SET_CMDFLAG(_bit, _flag) \
+   do { \
+      if (_flag) \
+         parsed_cmd->flags |= _bit; \
+   } while(0)
+
    parsed_cmd->output_level     = output_level;
    parsed_cmd->stats_types      = stats_work;
    parsed_cmd->sleep_strategy   = sleep_strategy_work;
-   parsed_cmd->timestamp_trace  = timestamp_trace_flag;
-   if (verify_flag)
-      parsed_cmd->verify_setvcp = true;
-   else if (noverify_flag)
-      parsed_cmd->verify_setvcp = false;
-   else
-      parsed_cmd->verify_setvcp = true;
-   parsed_cmd->nodetect         = nodetect_flag;
-   parsed_cmd->async            = async_flag;
-   parsed_cmd->report_freed_exceptions = report_freed_excp_flag;
-
-   if (notable_flag)
-      parsed_cmd->flags |= CMD_FLAG_NOTABLE;
-   if (show_unsupported_flag)
-      parsed_cmd->flags |= CMD_FLAG_SHOW_UNSUPPORTED;
-   if (force_flag)
-      parsed_cmd->flags |= CMD_FLAG_FORCE;
-
+   SET_CMDFLAG(CMD_FLAG_DDCDATA,           ddc_flag);
+   SET_CMDFLAG(CMD_FLAG_FORCE_SLAVE_ADDR,  force_slave_flag);
+   SET_CMDFLAG(CMD_FLAG_TIMESTAMP_TRACE,   timestamp_trace_flag);
+   SET_CMDFLAG(CMD_FLAG_VERIFY,            verify_flag || !noverify_flag);
+   // if (verify_flag || !noverify_flag)
+   //    parsed_cmd->flags |= CMD_FLAG_VERIFY;
+   SET_CMDFLAG(CMD_FLAG_NODETECT,          nodetect_flag);
+   SET_CMDFLAG(CMD_FLAG_ASYNC,             async_flag);
+   SET_CMDFLAG(CMD_FLAG_REPORT_FREED_EXCP, report_freed_excp_flag);
+   SET_CMDFLAG(CMD_FLAG_NOTABLE,           notable_flag);
+   SET_CMDFLAG(CMD_FLAG_SHOW_UNSUPPORTED,  show_unsupported_flag);
+   SET_CMDFLAG(CMD_FLAG_RW_ONLY,           rw_only_flag);
+   SET_CMDFLAG(CMD_FLAG_RO_ONLY,           ro_only_flag);
+   SET_CMDFLAG(CMD_FLAG_WO_ONLY,           wo_only_flag);
+   SET_CMDFLAG(CMD_FLAG_FORCE,             force_flag);
 
    if (failsim_fn_work) {
 #ifdef ENABLE_FAILSIM
-      parsed_cmd->enable_failure_simulation = true;
+      // parsed_cmd->enable_failure_simulation = true;
+      parsed_cmd->flags |= CMD_FLAG_ENABLE_FAILSIM;
       parsed_cmd->failsim_control_fn = failsim_fn_work;
 #else
       fprintf(stderr, "ddcutil not built with failure simulation support.  --failsim option invalid.\n");
@@ -362,6 +385,8 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 #endif
    }
 
+#undef SET_CMDFLAG
+
 
    // Create display identifier
    //
@@ -375,7 +400,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       int iDisplayIndex;
       bool adlok = parse_dot_separated_arg(adlwork, &iAdapterIndex, &iDisplayIndex);
       if (!adlok) {
-          printf("Invalid ADL argument: %s\n", adlwork );
+          fprintf(stderr, "Invalid ADL argument: %s\n", adlwork );
           ok = false;
           // DBGMSG("After ADL parse, ok=%d", ok);
       }
@@ -401,7 +426,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       if (!arg_ok)
          arg_ok = parse_colon_separated_arg(usbwork, &busnum, &devicenum);
       if (!arg_ok) {
-          printf("Invalid USB argument: %s\n", usbwork );
+          fprintf(stderr, "Invalid USB argument: %s\n", usbwork );
           ok = false;
           // DBGMSG("After USB parse, ok=%d", ok);
       }
@@ -516,10 +541,31 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
        }
        ntsa_free(pieces, /* free_strings */ true);
 
-       DBGMSF(debug, "retries = %d,%d,%d", parsed_cmd->max_tries[0], parsed_cmd->max_tries[1], parsed_cmd->max_tries[2]);
+       DBGMSF(debug, "retries = %d,%d,%d", parsed_cmd->max_tries[0],
+                                           parsed_cmd->max_tries[1],
+                                           parsed_cmd->max_tries[2]);
        debug = saved_debug;
     }
 
+   if (mccswork) {
+      DBGMSF(debug, "mccswork = |%s|", mccswork);
+      bool arg_ok = false;
+      DDCA_MCCS_Version_Spec vspec = parse_vspec(mccswork);
+      if (!vcp_version_eq(vspec, DDCA_VSPEC_UNKNOWN)) {
+         arg_ok = vcp_version_is_valid(vspec, false);
+      }
+      if (!arg_ok) {
+          fprintf(stderr, "Invalid MCCS spec: %s\n", mccswork );
+          ok = false;
+      }
+      else {
+         // which?
+         parsed_cmd->mccs_vspec = vspec;
+         parsed_cmd->mccs_version_id = mccs_version_spec_to_id(vspec);
+      }
+   }
+
+
 #ifdef COMMA_DELIMITED_TRACE
    if (tracework) {
        bool saved_debug = debug;
@@ -545,7 +591,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
                 traceClasses |= tg;
              }
              else {
-                printf("Invalid trace group: %s\n", token);
+                fprintf(stderr, "Invalid trace group: %s\n", token);
                 ok = false;
              }
           }
@@ -580,7 +626,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
                traceClasses |= tg;
             }
             else {
-               printf("Invalid trace group: %s\n", token);
+               fprintf(stderr, "Invalid trace group: %s\n", token);
                ok = false;
             }
         }
@@ -632,7 +678,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       puts("");
       // if no command specified, include license in version information and terminate
       if (rest_ct == 0) {
-         puts("Copyright (C) 2015-2017 Sanford Rockowitz");
+         puts("Copyright (C) 2015-2018 Sanford Rockowitz");
          puts("License GPLv2: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>");
          puts("This is free software: you are free to change and redistribute it.");
          puts("There is NO WARRANTY, to the extent permitted by law.");
@@ -675,14 +721,12 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
          int max_arg_ct = cmdInfo->max_arg_ct;
          int argctr = 1;
          while ( cmd_and_args[argctr] != NULL) {
-            // printf("loop.  argctr=%d\n", argctr);
             if (argctr > max_arg_ct) {
                fprintf(stderr, "Too many arguments\n");
                ok = false;
                break;
             }
             char * thisarg = (char *) cmd_and_args[argctr];
-            // printf("thisarg |%s|\n", thisarg);
             char * argcopy = strdup(thisarg);
             parsed_cmd->args[argctr-1] = argcopy;
             argctr++;
@@ -707,7 +751,33 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
             if (ok)
                parsed_cmd->fref = fsref;
             else
-               printf("Invalid feature code or subset: %s\n", parsed_cmd->args[0]);
+               fprintf(stderr, "Invalid feature code or subset: %s\n", parsed_cmd->args[0]);
+         }
+
+         // Ignore --notable for vcpinfo
+         if ( ok && parsed_cmd->cmd_id  == CMDID_VCPINFO) {
+            parsed_cmd->flags &= ~CMD_FLAG_NOTABLE;
+         }
+
+         if (ok && parsed_cmd->cmd_id == CMDID_GETVCP && (parsed_cmd->flags & CMD_FLAG_WO_ONLY) ) {
+            fprintf(stdout, "Ignoring option --wo-only\n");
+            parsed_cmd->flags &= ~CMD_FLAG_WO_ONLY;
+         }
+
+         if (ok && parsed_cmd->cmd_id == CMDID_SETVCP) {
+            if (parsed_cmd->argct == 3) {
+               if (streq(parsed_cmd->args[1],"+") || streq(parsed_cmd->args[1], "-")) {
+                  char * a1 = parsed_cmd->args[1];
+                  char * a2 = parsed_cmd->args[2];
+                  char * newarg1 = calloc(1, 1 + strlen(a2) + 1);
+                  strcpy(newarg1, a1);
+                  strcat(newarg1, a2);
+                  parsed_cmd->args[1] = newarg1;
+                  parsed_cmd->argct = 2;
+                  free(a1);
+                  free(a2);
+               }
+            }
          }
 
          // validate options vs commands
@@ -728,14 +798,18 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
          }
 #endif
 
-
       }  // recognized command
    }
 
    if (ok)
       ok = validate_output_level(parsed_cmd);
 
+   DBGMSF(debug, "Calling g_option_context_free(), context=%p...", context);
    g_option_context_free(context);
+   // if (trace_classes) {
+   //    DBGMSG("Freeing trace_classes=%p", trace_classes);
+   //    free(trace_classes);   // trying to avoid valgrind error re g_option_context_parse() - doesn't solve it
+   // }
 
    if (debug)
       dbgrpt_parsed_cmd(parsed_cmd, 0);
