@@ -62,6 +62,7 @@
 #include "usb/usb_displays.h"
 #endif
 
+#include "ddc/ddc_dynamic_features.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp.h"
 #include "ddc/ddc_vcp_version.h"
@@ -113,14 +114,14 @@ bool check_ddc_communication(Display_Handle * dh) {
 
    bool result = true;
 
-   DDCA_Single_Vcp_Value * pvalrec;
+   Single_Vcp_Value * pvalrec;
 
    // verbose output is distracting since this function is called when querying for other things
    // DDCA_Output_Level olev = get_output_level();
    // if (olev == DDCA_OL_VERBOSE)
    //    set_output_level(DDCA_OL_NORMAL);
 
-   Public_Status_Code psc = get_vcp_value(dh, 0x10, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
+   Public_Status_Code psc = ddc_get_vcp_value(dh, 0x10, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
 
    // if (olev == DDCA_OL_VERBOSE)
    //    set_output_level(olev);
@@ -165,14 +166,14 @@ bool check_monitor_ddc_null_response(Display_Handle * dh) {
 
    if (dh->dref->io_mode != DDCA_IO_USB) {
 
-      DDCA_Single_Vcp_Value * pvalrec;
+      Single_Vcp_Value * pvalrec;
 
       // // verbose output is distracting since this function is called when querying for other things
       // DDCA_Output_Level olev = get_output_level();
       // if (olev == DDCA_OL_VERBOSE)
       //    set_output_level(DDCA_OL_NORMAL);
 
-      Public_Status_Code psc = get_vcp_value(dh, 0x00, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
+      Public_Status_Code psc = ddc_get_vcp_value(dh, 0x00, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
 
       // if (olev == DDCA_OL_VERBOSE)
       //    set_output_level(olev);
@@ -266,16 +267,18 @@ bool initial_checks_by_dh(Display_Handle * dh) {
    bool debug = false;
    DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
    assert(dh);
-   DDCA_Single_Vcp_Value * pvalrec;
+   Single_Vcp_Value * pvalrec;
 
    if (!(dh->dref->flags & DREF_DDC_COMMUNICATION_CHECKED)) {
 
       Public_Status_Code psc = 0;
-      Error_Info * ddc_excp = get_vcp_value(dh, 0x00, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
+      Error_Info * ddc_excp = ddc_get_vcp_value(dh, 0x00, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
       DBGMSF(debug, "get_vcp_value() for feature 0x00 returned: %s", psc_desc(psc));
       if (psc == DDCRC_RETRIES && debug)
          DBGMSG("    Try errors: %s", errinfo_causes_string(ddc_excp));
+      if (ddc_excp)
+         errinfo_free(ddc_excp);
 
       if (psc == DDCRC_NULL_RESPONSE ||
           psc == DDCRC_ALL_RESPONSES_NULL ||
@@ -365,15 +368,15 @@ GPtrArray * ddc_get_all_displays() {
  * \remark
  * Consider caching the value in dh->dref
  */
-char * get_firmware_version_string(Display_Handle * dh) {
+static char * get_firmware_version_string_t(Display_Handle * dh) {
    bool debug = false;
 
-   static char version[40];
-
-   DDCA_Single_Vcp_Value * valrec;
+   static GPrivate  firmware_version_key = G_PRIVATE_INIT(g_free);
+   char * version = get_thread_fixed_buffer(&firmware_version_key, 40);
 
+   Single_Vcp_Value * valrec = NULL;
    Public_Status_Code psc = 0;
-   Error_Info * ddc_excp = get_vcp_value(
+   Error_Info * ddc_excp = ddc_get_vcp_value(
                                dh,
                                0xc9,                     // firmware detection
                                DDCA_NON_TABLE_VCP_VALUE,
@@ -387,8 +390,8 @@ char * get_firmware_version_string(Display_Handle * dh) {
       }
    }
    else {
-      snprintf(version, sizeof(version), "%d.%d",
-         valrec->val.nc.sh, valrec->val.nc.sl);
+      g_snprintf(version, 40, "%d.%d", valrec->val.nc.sh, valrec->val.nc.sl);
+      free_single_vcp_value(valrec);
    }
    return version;
 }
@@ -403,15 +406,19 @@ char * get_firmware_version_string(Display_Handle * dh) {
  * \remark
  * Consider caching the value in dh->dref
  */
-char * get_controller_mfg_string(Display_Handle * dh) {
+static char * get_controller_mfg_string_t(Display_Handle * dh) {
    bool debug = false;
 
-   static char mfg_name_buf[100] = "";
+   const int MFG_NAME_BUF_SIZE = 100;
+
+   static GPrivate  controller_mfg_key = G_PRIVATE_INIT(g_free);
+   char * mfg_name_buf = get_thread_fixed_buffer(&controller_mfg_key, MFG_NAME_BUF_SIZE);
+
    char * mfg_name = NULL;
-   DDCA_Single_Vcp_Value *   valrec;
+   Single_Vcp_Value * valrec;
 
    Public_Status_Code psc = 0;
-   Error_Info * ddc_excp = get_vcp_value(dh, 0xc8, DDCA_NON_TABLE_VCP_VALUE, &valrec);
+   Error_Info * ddc_excp = ddc_get_vcp_value(dh, 0xc8, DDCA_NON_TABLE_VCP_VALUE, &valrec);
    psc = (ddc_excp) ? ddc_excp->status_code : 0;
 
    if (psc == 0) {
@@ -420,9 +427,11 @@ char * get_controller_mfg_string(Display_Handle * dh) {
                             vals,
                             valrec->val.nc.sl);
       if (!mfg_name) {
-         snprintf(mfg_name_buf, sizeof(mfg_name_buf), "Unrecognized manufacturer code 0x%02x", valrec->val.nc.sl);
+         g_snprintf(mfg_name_buf, MFG_NAME_BUF_SIZE,
+                       "Unrecognized manufacturer code 0x%02x", valrec->val.nc.sl);
          mfg_name = mfg_name_buf;
       }
+      free_single_vcp_value(valrec);
    }
    else if (psc == DDCRC_REPORTED_UNSUPPORTED || psc == DDCRC_DETERMINED_UNSUPPORTED) {
       mfg_name = "Unspecified";
@@ -443,11 +452,15 @@ char * get_controller_mfg_string(Display_Handle * dh) {
  *
  * \param dref   pointer to display reference
  * \param depth  logical indentation depth
+ *
+ * \remark
+ * The detail level shown is controlled by the output level setting
+ * for the current thread.
  */
 void
 ddc_report_display_by_dref(Display_Ref * dref, int depth) {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
    assert(dref);
    assert(memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
    int d1 = depth+1;
@@ -464,10 +477,9 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
    }
 
    switch(dref->io_path.io_mode) {
-   case DDCA_IO_DEVI2C:
-      // i2c_report_active_display_by_busno(dref->io_path.io.i2c_busno, d1);
+   case DDCA_IO_I2C:
       {
-         I2C_Bus_Info * curinfo = dref->detail2;
+         I2C_Bus_Info * curinfo = dref->detail;
          assert(curinfo);
          assert(memcmp(curinfo, I2C_BUS_INFO_MARKER, 4) == 0);
 
@@ -521,22 +533,20 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
             }
             else {
                // display controller mfg, firmware version
-               rpt_vstring(d1, "Controller mfg:      %s", get_controller_mfg_string(dh) );
-               rpt_vstring(d1, "Firmware version:    %s", get_firmware_version_string(dh));;
+               rpt_vstring(d1, "Controller mfg:      %s", get_controller_mfg_string_t(dh) );
+               rpt_vstring(d1, "Firmware version:    %s", get_firmware_version_string_t(dh));;
                ddc_close_display(dh);
             }
 
             if (dref->io_path.io_mode != DDCA_IO_USB)
                rpt_vstring(d1, "Monitor returns DDC Null Response for unsupported features: %s",
-                                  bool_repr(dh->dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
+                                  bool_repr(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
          }
-
       }
-
    }
-   DBGMSF0(debug, "Done");
-}
 
+   DBGMSF(debug, "Done");
+}
 
 
 /** Reports all displays found.
@@ -544,14 +554,14 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
  * Output is written to the current report destination using
  * report functions.
  *
- * @param   valid_displays_only  if **true**, report only valid displays\n
- *                      if **false**, report all displays
+ * @param   include_invalid_displays  if false, report only valid displays\n
+ *                                    if true,  report all displays
  * @param   depth       logical indentation depth
  *
  * @return total number of displays reported
  */
 int
-ddc_report_displays(bool valid_displays_only, int depth) {
+ddc_report_displays(bool include_invalid_displays, int depth) {
    bool debug = false;
    DBGMSF0(debug, "Starting");
 
@@ -561,14 +571,14 @@ ddc_report_displays(bool valid_displays_only, int depth) {
    for (int ndx=0; ndx<all_displays->len; ndx++) {
       Display_Ref * dref = g_ptr_array_index(all_displays, ndx);
       assert(memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
-      if (dref->dispno > 0 || !valid_displays_only) {
+      if (dref->dispno > 0 || include_invalid_displays) {
          display_ct++;
          ddc_report_display_by_dref(dref, depth);
          rpt_title("",0);
       }
    }
    if (display_ct == 0)
-      rpt_vstring(depth, "No %sdisplays found", (valid_displays_only) ? "active " : "");
+      rpt_vstring(depth, "No %sdisplays found", (!include_invalid_displays) ? "active " : "");
 
    DBGMSF(debug, "Done.  Returning: %d", display_ct);
    return display_ct;
@@ -580,7 +590,7 @@ ddc_report_displays(bool valid_displays_only, int depth) {
  * \param dref  pointer to #Display_Ref
  * \param depth logical indentation depth
  */
-void dbgreport_display_ref(Display_Ref * dref, int depth) {
+void ddc_dbgrpt_display_ref(Display_Ref * dref, int depth) {
    int d1 = depth+1;
    int d2 = depth+2;
    // no longer needed for i2c_dbgreport_bus_info()
@@ -598,16 +608,16 @@ void dbgreport_display_ref(Display_Ref * dref, int depth) {
    rpt_vstring(d1, "io_mode: %s", io_mode_name(dref->io_path.io_mode));
    // rpt_vstring(d1, "flags:   0x%02x", drec->flags);
    switch(dref->io_path.io_mode) {
-   case(DDCA_IO_DEVI2C):
+   case(DDCA_IO_I2C):
          rpt_vstring(d1, "I2C bus information: ");
-         I2C_Bus_Info * businfo = dref->detail2;
+         I2C_Bus_Info * businfo = dref->detail;
          assert( memcmp(businfo->marker, I2C_BUS_INFO_MARKER, 4) == 0);
          i2c_dbgrpt_bus_info(businfo, d2);
          break;
    case(DDCA_IO_ADL):
 #ifdef HAVE_ADL
       rpt_vstring(d1, "ADL device information: ");
-      ADL_Display_Detail * adl_detail = dref->detail2;
+      ADL_Display_Detail * adl_detail = dref->detail;
       assert(memcmp(adl_detail->marker, ADL_DISPLAY_DETAIL_MARKER, 4) == 0);
       adlshim_report_adl_display_detail(adl_detail, d2);
 #endif
@@ -615,7 +625,7 @@ void dbgreport_display_ref(Display_Ref * dref, int depth) {
    case(DDCA_IO_USB):
 #ifdef USE_USB
          rpt_vstring(d1, "USB device information: ");
-         Usb_Monitor_Info * moninfo = dref->detail2;
+         Usb_Monitor_Info * moninfo = dref->detail;
          assert(memcmp(moninfo->marker, USB_MONITOR_INFO_MARKER, 4) == 0);
          report_usb_monitor_info(moninfo, d2);
 #else
@@ -633,14 +643,14 @@ void dbgreport_display_ref(Display_Ref * dref, int depth) {
  * \param recs    pointer to collection of #Display_Ref
  * \param depth   logical indentation depth
  */
-void debug_report_display_refs(GPtrArray * recs, int depth) {
+void ddc_dbgrpt_display_refs(GPtrArray * recs, int depth) {
    assert(recs);
    rpt_vstring(depth, "Reporting %d Display_Ref instances", recs->len);
    for (int ndx = 0; ndx < recs->len; ndx++) {
       Display_Ref * drec = g_ptr_array_index(recs, ndx);
       assert( memcmp(drec->marker, DISPLAY_REF_MARKER, 4) == 0);
       rpt_nl();
-      dbgreport_display_ref(drec, depth+1);
+      ddc_dbgrpt_display_ref(drec, depth+1);
    }
 }
 
@@ -703,7 +713,7 @@ ddc_check_display_ref(Display_Ref * dref, Display_Criteria * criteria) {
       goto bye;
 
    if (criteria->i2c_busno >= 0) {
-      if (dref->io_path.io_mode != DDCA_IO_DEVI2C || dref->io_path.path.i2c_busno != criteria->i2c_busno)
+      if (dref->io_path.io_mode != DDCA_IO_I2C || dref->io_path.path.i2c_busno != criteria->i2c_busno)
          goto bye;
    }
 
@@ -722,7 +732,7 @@ ddc_check_display_ref(Display_Ref * dref, Display_Criteria * criteria) {
          goto bye;
       char buf[40];
       snprintf(buf, 40, "%s/hiddev%d", usb_hiddev_directory(), criteria->hiddev);
-      Usb_Monitor_Info * moninfo = dref->detail2;
+      Usb_Monitor_Info * moninfo = dref->detail;
       assert(memcmp(moninfo->marker, USB_MONITOR_INFO_MARKER, 4) == 0);
       if (!streq( moninfo->hiddev_device_name, buf))
          goto bye;
@@ -777,7 +787,9 @@ bye:
 }
 
 
-
+/**
+ *  \param all_displays #GPtrArray of pointers to #Display_Ref
+ */
 void async_scan(GPtrArray * all_displays) {
    bool debug = false;
    DBGMSF(debug, "Starting. all_displays=%p, display_count=%d", all_displays, all_displays->len);
@@ -797,9 +809,10 @@ void async_scan(GPtrArray * all_displays) {
    DBGMSF(debug, "Started %d threads", threads->len);
    for (int ndx = 0; ndx < threads->len; ndx++) {
       GThread * thread = g_ptr_array_index(threads, ndx);
-      g_thread_join(thread);
+      g_thread_join(thread);  // implicitly unrefs the GThread
    }
    DBGMSF0(debug, "Threads joined");
+   g_ptr_array_free(threads, true);
 
 #ifdef OLD
    for (int ndx = 0; ndx < all_displays->len; ndx++) {
@@ -877,8 +890,8 @@ ddc_find_display_ref_by_display_identifier(Display_Identifier * did) {
 
    switch(did->id_type) {
    case DISP_ID_BUSNO:
-         criteria->i2c_busno = did->busno;
-         break;
+      criteria->i2c_busno = did->busno;
+      break;
    case DISP_ID_ADL:
       criteria->iAdapterIndex = did->iAdapterIndex;
       criteria->iDisplayIndex = did->iDisplayIndex;
@@ -916,7 +929,7 @@ ddc_find_display_ref_by_display_identifier(Display_Identifier * did) {
    if (debug) {
       if (result) {
          DBGMSG0("Done.  Returning: ");
-         dbgreport_display_ref(result, 1);
+         ddc_dbgrpt_display_ref(result, 1);
       }
       else
          DBGMSG0("Done.  Returning NULL");
@@ -945,7 +958,7 @@ get_display_ref_for_display_identifier(
 {
    Display_Ref * dref = ddc_find_display_ref_by_display_identifier(pdid);
    if ( !dref && (callopts & CALLOPT_ERR_MSG) ) {
-      f0printf(FERR, "Display not found\n");
+      f0printf(ferr(), "Display not found\n");
    }
 
    return dref;
@@ -973,7 +986,7 @@ ddc_detect_all_displays() {
          dref->dispno = -1;
          dref->pedid = businfo->edid;    // needed?
          // drec->detail.bus_detail = businfo;
-         dref->detail2 = businfo;
+         dref->detail = businfo;
          dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
          dref->flags |= DREF_DDC_IS_MONITOR;
          g_ptr_array_add(display_list, dref);
@@ -988,11 +1001,15 @@ ddc_detect_all_displays() {
      dref->dispno = -1;
      dref->pedid = detail->pEdid;   // needed?
      // drec->detail.adl_detail = detail;
-     dref->detail2 = detail;
+     dref->detail = detail;
      dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
      dref->flags |= DREF_DDC_IS_MONITOR;
      g_ptr_array_add(display_list, dref);
   }
+  // Unlike businfo and usb_monitors, which point to persistent data structures,
+  // all_adl_details points to a transitory data structure created by
+  // adlshim_get_valid_display_details() and must be freed
+  g_ptr_array_free(all_adl_details, true);
 
 #ifdef USE_USB
    GPtrArray * usb_monitors = get_usb_monitor_list();
@@ -1007,7 +1024,7 @@ ddc_detect_all_displays() {
       dref->dispno = -1;
       dref->pedid = curmon->edid;
       // drec->detail.usb_detail = curmon;
-      dref->detail2 = curmon;
+      dref->detail = curmon;
       dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
       dref->flags |= DREF_DDC_IS_MONITOR;
       g_ptr_array_add(display_list, dref);
@@ -1015,12 +1032,15 @@ ddc_detect_all_displays() {
 #endif
 
    // verbose output is distracting within scans
-   // saved and reset here to that async threads are not adjusting output level
+   // saved and reset here so that async threads are not adjusting output level
    DDCA_Output_Level olev = get_output_level();
    if (olev == DDCA_OL_VERBOSE)
       set_output_level(DDCA_OL_NORMAL);
 
-   if (display_list->len >= async_threshold && all_adl_details->len == 0)
+   DBGMSF(debug, "display_list->len=%d, async_threshold=%d, adlct=%d",
+                 display_list->len, async_threshold, adlct);
+   // ADL displays do not support async scan.  Not worth implementing.
+   if (display_list->len >= async_threshold && adlct == 0)
    // if (true)
       async_scan(display_list);
    else
@@ -1035,6 +1055,9 @@ ddc_detect_all_displays() {
       assert( memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0 );
       if (dref->flags & DREF_DDC_COMMUNICATION_WORKING) {
          dref->dispno = ++dispno_max;
+
+         // check_dynamic_features(dref);    // wrong location for hook
+
       }
       else {
          dref->dispno = -1;
