@@ -65,6 +65,7 @@
 #include "usb/usb_displays.h"
 #endif
 
+#include "ddc/ddc_display_lock.h"
 #include "ddc/ddc_try_stats.h"
 
 #include "ddc/ddc_packet_io.h"
@@ -110,12 +111,12 @@ bool is_ddc_null_message(Byte * packet) {
  *  \param  dref            display reference
  *  \param  callopts        call option flags
  *  \param  pdh             address at which to return display handle
+ *  \return status code     as from #i2c_open_bus(), #usb_open_hiddev_device()
+ *  \retval DDCRC_LOCKED    display open in another thread
  *
- * Returns:
- *    status code
- *
- * Notes:
- *    Will abort if open fails and CALLOPT_ERR_ABORT set
+ *  **Call_Option** flags recognized:
+ *  - CALLOPT_WAIT
+ *  - CALLOPT_ERR_MSG
  */
 Public_Status_Code ddc_open_display(
       Display_Ref *    dref,
@@ -129,12 +130,23 @@ Public_Status_Code ddc_open_display(
    Display_Handle * dh = NULL;
    Public_Status_Code psc = 0;
 
+   Distinct_Display_Ref display_id = get_distinct_display_ref(dref);
+   Distinct_Display_Flags ddisp_flags = DDISP_NONE;
+   if (callopts & CALLOPT_WAIT)
+      ddisp_flags |= DDISP_WAIT;
+
+   bool locked = lock_distinct_display(display_id, ddisp_flags);
+   if (!locked) {
+      psc = DDCRC_LOCKED;          // is there an appropriate errno value?  EBUSY? EACCES?
+      goto bye;
+   }
+
    switch (dref->io_path.io_mode) {
 
-   case DDCA_IO_DEVI2C:
+   case DDCA_IO_I2C:
       {
          int fd = i2c_open_bus(dref->io_path.path.i2c_busno, callopts);
-         if (fd < 0) {    // will be < 0 if open_i2c_bus failed and CALLOPT_ERR_ABORT not set
+         if (fd < 0) {
             psc = fd;
             goto bye;
          }
@@ -151,27 +163,26 @@ Public_Status_Code ddc_open_display(
          // Is this needed?
          // 10/24/15, try disabling:
          // sleepMillisWithTrace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, NULL);
+
          dh = create_bus_display_handle_from_display_ref(fd, dref);    // n. sets dh->dref = dref
-         // n. sets
-         // Bus_Info * bus_info = i2c_get_bus_info(dref->io_path.path.i2c_busno, DISPSEL_VALID_ONLY);   // or DISPSEL_NONE?
-         // Bus_Info * bus_info = i2c_get_bus_info_new(dref->io_path.path.i2c_busno);   // or DISPSEL_NONE?
-         I2C_Bus_Info * bus_info = dref->detail2;
+
+         I2C_Bus_Info * bus_info = dref->detail;
          assert(bus_info);   // need to convert to a test?
          assert( memcmp(bus_info, I2C_BUS_INFO_MARKER, 4) == 0);
-         dref->pedid = bus_info->edid;
 
+         dref->pedid = bus_info->edid;
          if (!dref->pedid) {
             // How is this even possible?
             // 1/2017:  Observed with x260 laptop and Ultradock, See ddcutil user report.
             //          close(fd) fails
             DBGMSG("No EDID for device on bus /dev/i2c-%d", dref->io_path.path.i2c_busno);
-            if (!(callopts & CALLOPT_FORCE)) {
+            // if (!(callopts & CALLOPT_FORCE)) {
                close(fd);
                psc = DDCRC_EDID;
                goto bye;
-            }
-            else
-               DBGMSG0("Continuing");
+            // }
+            // else
+            //    DBGMSG0("Continuing");
          }
       }
       break;
@@ -208,12 +219,17 @@ Public_Status_Code ddc_open_display(
    // sleep_millis_with_trace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, NULL);
    if (dref->io_path.io_mode != DDCA_IO_USB)
       call_tuned_sleep_i2c(SE_POST_OPEN);
-   // report_display_handle(pDispHandle, __func__);
+   // dbgrpt_display_handle(dh, __func__, 1);
+
 bye:
-   if (psc != 0)
+   if (psc != 0) {
+      if (locked)
+         unlock_distinct_display(display_id);
       COUNT_STATUS_CODE(psc);
+   }
    *pdh = dh;
    assert(psc <= 0);
+   // dbgrpt_distinct_display_descriptors(0);
    DBGMSF(debug, "Done.  Returning: %s, *pdh=%d", psc_desc(psc), *pdh);
    return psc;
 }
@@ -234,7 +250,7 @@ void ddc_close_display(Display_Handle * dh) {
    }
 
    switch(dh->dref->io_path.io_mode) {
-   case DDCA_IO_DEVI2C:
+   case DDCA_IO_I2C:
       {
          Status_Errno rc = i2c_close_bus(dh->fh, dh->dref->io_path.path.i2c_busno,  CALLOPT_NONE);    // return error if failure
          if (rc != 0) {
@@ -265,6 +281,9 @@ void ddc_close_display(Display_Handle * dh) {
 #endif
    } //switch
 
+   Distinct_Display_Ref display_id = get_distinct_display_ref(dh->dref);
+   unlock_distinct_display(display_id);
+
    free_display_handle(dh);
 }
 
@@ -386,7 +405,7 @@ static Public_Status_Code ddc_i2c_write_read_raw(
 
    assert(dh);
    assert(dh->dref);
-   assert(dh->dref->io_path.io_mode == DDCA_IO_DEVI2C);
+   assert(dh->dref->io_path.io_mode == DDCA_IO_I2C);
    // ASSERT_DISPLAY_IO_MODE(dh, DDCA_IO_DEVI2C);
 
 #ifdef TEST_THAT_DIDNT_WORK
@@ -489,8 +508,7 @@ static Public_Status_Code ddc_adl_write_read_raw(
          else if (memcmp(get_packet_start(request_packet_ptr), readbuf, get_packet_len(request_packet_ptr)) == 0) {
             // is this a DDC error or a programming bug?
             DDCMSG(debug, "Bytes read same as bytes written.", __func__ );
-            psc = DDCRC_READ_EQUALS_WRITE;
-            COUNT_STATUS_CODE(psc);
+            psc = COUNT_STATUS_CODE(DDCRC_DDC_DATA);   // was DDCRC_READ_EQUALS_WRITE
          }
          else {
             psc = 0;
@@ -523,9 +541,9 @@ static Public_Status_Code ddc_write_read_raw(
    Public_Status_Code psc;
 
    // This function should not be called for USB
-   assert(dh->dref->io_path.io_mode == DDCA_IO_DEVI2C || dh->dref->io_path.io_mode == DDCA_IO_ADL);
+   assert(dh->dref->io_path.io_mode == DDCA_IO_I2C || dh->dref->io_path.io_mode == DDCA_IO_ADL);
 
-   if (dh->dref->io_path.io_mode == DDCA_IO_DEVI2C) {
+   if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
         psc =  ddc_i2c_write_read_raw(
               dh,
               request_packet_ptr,
@@ -720,7 +738,7 @@ ddc_write_read_with_retry(
          DBGMSF(debug, "ddc_write_read() returned %s", psc_desc(psc) );
          COUNT_RETRYABLE_STATUS_CODE(psc);
 
-         if (dh->dref->io_path.io_mode == DDCA_IO_DEVI2C) {
+         if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
             // The problem: Does NULL response indicate an error condition, or
             // is the monitor using NULL response to indicate unsupported?
             // Acer monitor uses NULL response instead of setting the unsupported
@@ -731,7 +749,7 @@ ddc_write_read_with_retry(
                DBGMSF(debug, "DDCRC_NULL_RESPONSE, retryable=%s", bool_repr(retryable));
                if (retryable) {
                   if (ddcrc_null_response_ct == 1 && get_output_level() >= DDCA_OL_VERBOSE)
-                     f0printf(FOUT, "Extended delay as recovery from DDC Null Response...\n");
+                     f0printf(fout(), "Extended delay as recovery from DDC Null Response...\n");
                   call_dynamic_tuned_sleep_i2c(SE_DDC_NULL, ddcrc_null_response_ct);
                }
             }
@@ -867,7 +885,7 @@ ddc_write_only(
 
    Public_Status_Code psc = 0;
    assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
-   if (dh->dref->io_path.io_mode == DDCA_IO_DEVI2C) {
+   if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
       psc = ddc_i2c_write_only(dh->fh, request_packet_ptr);
    }
    else {
@@ -933,7 +951,7 @@ ddc_write_only_with_retry(
 
       if (psc < 0) {
          COUNT_RETRYABLE_STATUS_CODE(psc);
-         if (dh->dref->io_path.io_mode == DDCA_IO_DEVI2C) {
+         if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
             if (psc < 0) {
                if (psc != -EIO)
                    retryable = false;
