@@ -70,6 +70,32 @@
 static Trace_Group TRACE_GROUP = TRC_DDC;
 
 
+
+typedef struct {
+   bool   verify_setvcp;
+} Thread_Vcp_Settings;
+
+static Thread_Vcp_Settings *  get_thread_vcp_settings() {
+   static GPrivate per_thread_key = G_PRIVATE_INIT(g_free);
+
+   Thread_Vcp_Settings *settings = g_private_get(&per_thread_key);
+
+   // GThread * this_thread = g_thread_self();
+   // printf("(%s) this_thread=%p, settings=%p\n", __func__, this_thread, settings);
+
+   if (!settings) {
+      settings = g_new0(Thread_Vcp_Settings, 1);
+      settings->verify_setvcp = false;     // set by g_new0(), but be explicit
+
+      g_private_set(&per_thread_key, settings);
+   }
+
+   // printf("(%s) Returning: %p\n", __func__, settings);
+   return settings;
+}
+
+
+
 //
 //  Save Control Settings
 //
@@ -80,7 +106,7 @@ static Trace_Group TRACE_GROUP = TRC_DDC;
  * \return NULL if success, pointer to #Ddc_Error if failure
  */
 Error_Info *
-save_current_settings(
+ddc_save_current_settings(
       Display_Handle * dh)
 {
    bool debug = false;
@@ -92,7 +118,7 @@ save_current_settings(
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
       // command line parser should block this case
       PROGRAM_LOGIC_ERROR("MCCS over USB does not have Save Current Settings command", NULL);
-      ddc_excp = errinfo_new(DDCL_UNIMPLEMENTED, __func__);
+      ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__);
    }
    else {
       DDC_Packet * request_packet_ptr =
@@ -127,7 +153,7 @@ save_current_settings(
  *          pointer to #Ddc_Error from #ddc_write_only_with_retry() if failure
  */
 Error_Info *
-set_nontable_vcp_value(
+ddc_set_nontable_vcp_value(
       Display_Handle * dh,
       Byte             feature_code,
       int              new_value)
@@ -191,7 +217,7 @@ set_table_vcp_value(
 
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
 #ifdef USE_USB
-      psc = DDCL_UNIMPLEMENTED;
+      psc = DDCRC_UNIMPLEMENTED;
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
 #endif
@@ -214,30 +240,32 @@ set_table_vcp_value(
 }
 
 
-
-static bool verify_setvcp = false;
-
-/** Sets the setvcp verification setting.
+/** Sets the setvcp verification setting for the current thread.
  *
  *  If enabled, setvcp will read the feature value from the monitor after
  *  writing it, to ensure the monitor has actually changed the feature value.
  *
  *  \param onoff  **true** for enabled, **false** for disabled.
+ *  \return prior setting
  */
-void set_verify_setvcp(bool onoff) {
+bool ddc_set_verify_setvcp(bool onoff) {
    bool debug = false;
    DBGMSF(debug, "Setting verify_setvcp = %s", bool_repr(onoff));
-   verify_setvcp = onoff;
+   Thread_Vcp_Settings * settings = get_thread_vcp_settings();
+   bool old_value = settings->verify_setvcp;
+   settings->verify_setvcp = onoff;
+   return old_value;
 }
 
 
-/** Gets the current setvcp verification setting.
+/** Gets the current setvcp verification setting for the current thread.
  *
  *  \return **true** if setvcp verification enabled\n
  *          **false** if not
  */
-bool get_verify_setvcp() {
-   return verify_setvcp;
+bool ddc_get_verify_setvcp() {
+   Thread_Vcp_Settings * settings = get_thread_vcp_settings();
+   return settings->verify_setvcp;
 }
 
 
@@ -264,9 +292,9 @@ is_rereadable_feature(
       DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);  // ensure dh->vcp_version set
       DBGMSF(debug, "vspec = %d.%d", vspec.major, vspec.minor);
       // hack, make a guess
-      if ( vcp_version_eq(vspec, VCP_SPEC_UNKNOWN)   ||
-           vcp_version_eq(vspec, VCP_SPEC_UNQUERIED ))
-         vspec = VCP_SPEC_V22;
+      if ( vcp_version_eq(vspec, DDCA_VSPEC_UNKNOWN)   ||
+           vcp_version_eq(vspec, DDCA_VSPEC_UNQUERIED ))
+         vspec = DDCA_VSPEC_V22;
 
       // if ( !vcp_version_eq(vspec, VCP_SPEC_UNKNOWN) &&
       //      !vcp_version_eq(vspec, VCP_SPEC_UNQUERIED ))
@@ -292,8 +320,8 @@ is_rereadable_feature(
 
 static bool
 single_vcp_value_equal(
-      DDCA_Single_Vcp_Value * vrec1,
-      DDCA_Single_Vcp_Value * vrec2)
+      Single_Vcp_Value * vrec1,
+      Single_Vcp_Value * vrec2)
 {
    assert(vrec1 && vrec2);  // no implementation for degenerate cases
    bool debug = false;
@@ -325,26 +353,35 @@ single_vcp_value_equal(
  *
  *  \param  dh            display handle for open display
  *  \param  vrec          pointer to value record
+ *  \param  newval_loc    if non-null, address at which to return verified value
  *  \return NULL if success, pointer to #Ddc_Error if failure
  *
  *  If write verification is turned on, reads the feature value after writing it
  *  to ensure the display has actually changed the value.
+ *
+ * The caller is responsible for freeing the value returned at **newval_loc**.
+ *  \remark
+ *  If verbose messages are in effect, writes detailed messages to the current
+ *  stdout device.
  */
 Error_Info *
-set_vcp_value(
-      Display_Handle *        dh,
-      DDCA_Single_Vcp_Value * vrec)
+ddc_set_vcp_value(
+      Display_Handle *    dh,
+      Single_Vcp_Value *  vrec,
+      Single_Vcp_Value ** newval_loc)
 {
    bool debug = false;
    DBGMSF0(debug, "Starting. ");
-   FILE * fout = FOUT;
-   if ( get_output_level() < DDCA_OL_VERBOSE )
-      fout = NULL;
+   FILE * verbose_msg_dest = fout();
+   if ( get_output_level() < DDCA_OL_VERBOSE && !debug )
+      verbose_msg_dest = NULL;
 
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
+   if (newval_loc)
+      *newval_loc = NULL;
    if (vrec->value_type == DDCA_NON_TABLE_VCP_VALUE) {
-      ddc_excp = set_nontable_vcp_value(dh, vrec->opcode, vrec->val.c.cur_val);
+      ddc_excp = ddc_set_nontable_vcp_value(dh, vrec->opcode, vrec->val.c.cur_val);
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
    }
    else {
@@ -353,38 +390,44 @@ set_vcp_value(
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
    }
 
-   if (!ddc_excp && verify_setvcp) {
+   if (!ddc_excp && ddc_get_verify_setvcp()) {
       if (is_rereadable_feature(dh, vrec->opcode) ) {
-         f0printf(fout, "Verifying that value of feature 0x%02x successfully set...\n", vrec->opcode);
-         DDCA_Single_Vcp_Value * newval = NULL;
-         ddc_excp = get_vcp_value(
+         f0printf(verbose_msg_dest, "Verifying that value of feature 0x%02x successfully set...\n", vrec->opcode);
+         Single_Vcp_Value * newval = NULL;
+         ddc_excp = ddc_get_vcp_value(
              dh,
              vrec->opcode,
              vrec->value_type,
              &newval);
          psc = (ddc_excp) ? ddc_excp->status_code : 0;
          if (ddc_excp) {
-            f0printf(fout, "(%s) Read after write failed. get_vcp_value() returned: %s\n",
+            f0printf(verbose_msg_dest, "(%s) Read after write failed. get_vcp_value() returned: %s\n",
                            __func__, psc_desc(psc));
             if (psc == DDCRC_RETRIES)
-               f0printf(fout, "(%s)    Try errors: %s\n", __func__, errinfo_causes_string(ddc_excp));
+               f0printf(verbose_msg_dest, "(%s)    Try errors: %s\n", __func__, errinfo_causes_string(ddc_excp));
             // psc = DDCRC_VERIFY;
          }
          else {
             assert(vrec && newval);    // silence clang complaint
+            // dbgrpt_ddca_single_vcp_value(vrec, 2);
+            // dbgrpt_ddca_single_vcp_value(newval, 3);
+
             if (! single_vcp_value_equal(vrec,newval)) {
                psc = DDCRC_VERIFY;
                ddc_excp = errinfo_new(DDCRC_VERIFY, __func__);
-               f0printf(fout, "Current value does not match value set.\n");
+               f0printf(verbose_msg_dest, "Current value does not match value set.\n");
             }
             else {
-               f0printf(fout, "Verification succeeded\n");
+               f0printf(verbose_msg_dest, "Verification succeeded\n");
             }
+            if (newval_loc)
+               *newval_loc = newval;
+            else
+               free_single_vcp_value(newval);
          }
       }
       else {
-         f0printf(fout, "Feature 0x%02x does not support verification\n", vrec->opcode);
-         // rpt_vstring(0, "Feature 0x%02x does not support verification", vrec->opcode);
+         f0printf(verbose_msg_dest, "Feature 0x%02x does not support verification\n", vrec->opcode);
       }
    }
 
@@ -409,7 +452,7 @@ set_vcp_value(
  * The value pointed to by ppInterpretedCode is non-null iff the returned status code is 0.
  */
 Error_Info *
-get_nontable_vcp_value(
+ddc_get_nontable_vcp_value(
        Display_Handle *               dh,
        DDCA_Vcp_Feature_Code          feature_code,
        Parsed_Nontable_Vcp_Response** ppInterpretedCode)
@@ -442,9 +485,11 @@ get_nontable_vcp_value(
            false,                       // all_zero_response_ok
            &response_packet_ptr
         );
+   assert( (!excp && response_packet_ptr) || (excp && !response_packet_ptr));
    if (debug || IS_TRACING() ) {
       if (psc != 0)
-         DBGMSG("perform_ddc_write_read_with_retry() returned %s, reponse_packet_ptr=%p", psc_desc(psc), response_packet_ptr);
+         DBGMSG("perform_ddc_write_read_with_retry() returned %s, reponse_packet_ptr=%p",
+                psc_desc(psc), response_packet_ptr);
    }
    // psc = (excp) ? excp->psc : 0;
 
@@ -463,13 +508,14 @@ get_nontable_vcp_value(
 #endif
 
          if (!parsed_response->valid_response)  {
-            psc = DDCRC_INVALID_DATA;
+            psc = DDCRC_DDC_DATA;             // was DDCRC_INVALID_DATA
             excp = errinfo_new(psc, __func__);
          }
          else if (!parsed_response->supported_opcode) {
             psc = DDCRC_REPORTED_UNSUPPORTED;
             excp = errinfo_new(psc, __func__);
          }
+
          if (psc != 0) {
             free(parsed_response);
             parsed_response = NULL;
@@ -485,7 +531,7 @@ get_nontable_vcp_value(
    if (response_packet_ptr)
       free_ddc_packet(response_packet_ptr);
 
-
+   assert( (!excp && parsed_response) || (excp && !parsed_response)); // needed to avoid clang warning
    if (debug || IS_TRACING() ) {
       if (excp) {
          DBGMSG("Error reading feature x%02x.  Returning exception: ", feature_code);
@@ -493,10 +539,12 @@ get_nontable_vcp_value(
       }
       else {
          DBGMSG("Success reading feature x%02x. *ppinterpreted_code=%p", feature_code, parsed_response);
+         DBGMSG("  mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x",
+                parsed_response->mh, parsed_response->ml, parsed_response->sh, parsed_response->sl);
       }
    }
    *ppInterpretedCode = parsed_response;
-   assert( (!excp && parsed_response) || (excp && !parsed_response));
+
    return excp;
 }
 
@@ -509,7 +557,7 @@ get_nontable_vcp_value(
  *  \param  pp_table_bytes  location at which to save address of newly allocated Buffer
  *  \return NULL if success, pointer to #Ddc_Error if failure
  */
-Error_Info * get_table_vcp_value(
+Error_Info * ddc_get_table_vcp_value(
        Display_Handle *       dh,
        Byte                   feature_code,
        Buffer**               pp_table_bytes)
@@ -554,19 +602,19 @@ Error_Info * get_table_vcp_value(
  * \param  dh              handle for open display
  * \param  feature_code    feature code id
  * \param  call_type       indicates whether table or non-table
- * \param  pvalrec         location where to return newly allocated #DDCA_Single_Vcp_Value
+ * \param  pvalrec         location where to return newly allocated #Single_Vcp_Value
  * \return NULL if success, pointer to #Ddc_Error if failure
  *
  * The value pointed to by pvalrec is non-null iff the returned status code is 0.
  *
- * The caller is responsible for freeing the value pointed returned at pvalrec.
+ * The caller is responsible for freeing the value returned at **valrec_loc**.
  */
 Error_Info *
-get_vcp_value(
-       Display_Handle *          dh,
-       Byte                      feature_code,
-       DDCA_Vcp_Value_Type       call_type,
-       DDCA_Single_Vcp_Value **  pvalrec)
+ddc_get_vcp_value(
+       Display_Handle *       dh,
+       Byte                   feature_code,
+       DDCA_Vcp_Value_Type    call_type,
+       Single_Vcp_Value **    valrec_loc)
 {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting. Reading feature 0x%02x, dh=%s, dh->fh=%d",
@@ -576,7 +624,7 @@ get_vcp_value(
    Error_Info * ddc_excp = NULL;
    Buffer * buffer = NULL;
    Parsed_Nontable_Vcp_Response * parsed_nontable_response = NULL;  // vs interpreted ..
-   DDCA_Single_Vcp_Value * valrec = NULL;
+   Single_Vcp_Value * valrec = NULL;
 
    // why are we coming here for USB?
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
@@ -602,8 +650,8 @@ get_vcp_value(
                 break;
 
           case (DDCA_TABLE_VCP_VALUE):
-                psc = DDCL_UNIMPLEMENTED;
-                ddc_excp = errinfo_new(DDCL_UNIMPLEMENTED, __func__);
+                psc = DDCRC_UNIMPLEMENTED;
+                ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__);
                 break;
           }
 #else
@@ -614,7 +662,7 @@ get_vcp_value(
       switch (call_type) {
 
       case (DDCA_NON_TABLE_VCP_VALUE):
-            ddc_excp = get_nontable_vcp_value(
+            ddc_excp = ddc_get_nontable_vcp_value(
                           dh,
                           feature_code,
                           &parsed_nontable_response);
@@ -631,7 +679,7 @@ get_vcp_value(
             break;
 
       case (DDCA_TABLE_VCP_VALUE):
-            ddc_excp = get_table_vcp_value(
+            ddc_excp = ddc_get_table_vcp_value(
                     dh,
                     feature_code,
                     &buffer);
@@ -645,13 +693,108 @@ get_vcp_value(
 
    } // non USB
 
-   *pvalrec = valrec;
+   *valrec_loc = valrec;
 
    DBGTRC(debug, TRACE_GROUP, "Done. psc=%s", psc_desc(psc) );
    if (psc == 0 && debug)
       report_single_vcp_value(valrec,1);
-   assert( (psc == 0 && *pvalrec) || (psc != 0 && !*pvalrec) );
+   assert( (psc == 0 && *valrec_loc) || (psc != 0 && !*valrec_loc) );
    DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
    return ddc_excp;
 }
 
+#ifdef ALT
+Error_Info *
+ddc_get_vcp_value2(
+       Display_Handle *       dh,
+       Byte                   feature_code,
+       DDCA_Vcp_Value_Type    call_type,
+       DDCA_Any_Vcp_Value **  valrec_loc)
+{
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "Starting. Reading feature 0x%02x, dh=%s, dh->fh=%d",
+            feature_code, dh_repr_t(dh), dh->fh);
+
+   Public_Status_Code psc = 0;
+   Error_Info * ddc_excp = NULL;
+   Buffer * buffer = NULL;
+   Parsed_Nontable_Vcp_Response * parsed_nontable_response = NULL;  // vs interpreted ..
+   DDCA_Any_Vcp_Value * valrec = NULL;
+
+   // why are we coming here for USB?
+   if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
+#ifdef USE_USB
+      DBGMSF0(debug, "USB case");
+
+      switch (call_type) {
+
+          case (DDCA_NON_TABLE_VCP_VALUE):
+                psc = usb_get_nontable_vcp_value(
+                      dh,
+                      feature_code,
+                      &parsed_nontable_response);    //
+                if (psc == 0) {
+                   valrec = create_nontable_vcp_value2(
+                               feature_code,
+                               parsed_nontable_response->mh,
+                               parsed_nontable_response->ml,
+                               parsed_nontable_response->sh,
+                               parsed_nontable_response->sl);
+                   free(parsed_nontable_response);
+                }
+                break;
+
+          case (DDCA_TABLE_VCP_VALUE):
+                psc = DDCRC_UNIMPLEMENTED;
+                ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__);
+                break;
+          }
+#else
+      PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
+#endif
+   }
+   else {
+      switch (call_type) {
+
+      case (DDCA_NON_TABLE_VCP_VALUE):
+            ddc_excp = ddc_get_nontable_vcp_value(
+                          dh,
+                          feature_code,
+                          &parsed_nontable_response);
+            psc = (ddc_excp) ? ddc_excp->status_code : 0;
+            if (!ddc_excp) {
+               valrec = create_nontable_vcp_value2(
+                           feature_code,
+                           parsed_nontable_response->mh,
+                           parsed_nontable_response->ml,
+                           parsed_nontable_response->sh,
+                           parsed_nontable_response->sl);
+               free(parsed_nontable_response);
+            }
+            break;
+
+      case (DDCA_TABLE_VCP_VALUE):
+            ddc_excp = ddc_get_table_vcp_value(
+                    dh,
+                    feature_code,
+                    &buffer);
+            psc = ERRINFO_STATUS(ddc_excp);
+            if (!ddc_excp) {
+               valrec = create_table_vcp_value_by_buffer2(feature_code, buffer);
+               buffer_free(buffer, __func__);
+            }
+            break;
+      }
+
+   } // non USB
+
+   *valrec_loc = valrec;
+
+   DBGTRC(debug, TRACE_GROUP, "Done. psc=%s", psc_desc(psc) );
+   if (psc == 0 && debug)
+      report_any_vcp_value(valrec,1);
+   assert( (psc == 0 && *valrec_loc) || (psc != 0 && !*valrec_loc) );
+   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
+   return ddc_excp;
+}
+#endif
