@@ -83,7 +83,10 @@ bool i2c_force_slave_addr_flag = false;
  *  @param callopts  call option flags, controlling failure action
  *
  *  @retval >=0  file descriptor
- *  @retval -errno  negative Linux errno if open fails and CALLOPT_ERR_ABORT not set in callopts
+ *  @retval -errno  negative Linux errno if open fails
+ *
+ *  Call options recognized
+ *  - CALLOPT_ERR_MSG
  */
 int i2c_open_bus(int busno, Byte callopts) {
    bool debug = false;
@@ -109,7 +112,7 @@ int i2c_open_bus(int busno, Byte callopts) {
       }
 #endif
       if (callopts & CALLOPT_ERR_MSG) {
-         f0printf(FERR, "Open failed for %s: errno=%s\n",
+         f0printf(ferr(), "Open failed for %s: errno=%s\n",
                         filename, linux_errno_desc(errsv));
       }
       file = -errsv;
@@ -168,7 +171,7 @@ Status_Errno i2c_close_bus(int fd, int busno, Call_Options callopts) {
          TERMINATE_EXECUTION_ON_ERROR(workbuf);
 #endif
       if (callopts & CALLOPT_ERR_MSG)
-         f0printf(FERR, "%s\n", workbuf);
+         f0printf(ferr(), "%s\n", workbuf);
 
       result = -errsv;
    }
@@ -577,7 +580,7 @@ Public_Status_Code i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid) {
          if (rc == 0) {
             rawedid->len = 128;
             if (debug) {
-               DBGMSG("call_read returned:", NULL);
+               DBGMSG("call_read returned:");
                dbgrpt_buffer(rawedid, 1);
                DBGMSG("edid checksum = %d", edid_checksum(rawedid->bytes) );
             }
@@ -588,7 +591,7 @@ Public_Status_Code i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid) {
                // e.g. nouveau driver, Quadro card, on blackrock
                DBGTRC(debug, TRACE_GROUP, "Invalid EDID checksum %d, expected 0.", checksum);
                rawedid->len = 0;
-               rc = DDCRC_EDID;
+               rc = DDCRC_INVALID_EDID;    // was DDCRC_EDID
             }
          }
          if (rc == 0)
@@ -634,9 +637,9 @@ Public_Status_Code i2c_get_parsed_edid_by_fd(int fd, Parsed_Edid ** edid_ptr_loc
             DBGMSG("create_parsed_edid() returned NULL", NULL);
       }
       if (!edid)
-         rc = DDCRC_EDID;
+         rc = DDCRC_INVALID_EDID;
    }
-   else if (rc == DDCRC_EDID) {
+   else {        // if (rc == DDCRC_EDID) {
       DBGTRC(debug, TRACE_GROUP, "i2c_get_raw_edid_by_fd() returned %s", psc_desc(rc));
    }
 
@@ -697,7 +700,7 @@ static void i2c_check_bus(I2C_Bus_Info * bus_info) {
             Status_Errno_DDC psc = i2c_detect_ddc_addrs_by_fd(file, &ddc_addr_flags);
             if (psc != 0) {
                DBGMSF(debug, "detect_ddc_addrs_by_fd() returned %d", psc);
-               f0printf(FERR, "Failure detecting bus addresses for /dev/i2c-%d: status code=%s\n",
+               f0printf(ferr(), "Failure detecting bus addresses for /dev/i2c-%d: status code=%s\n",
                               bus_info->busno, psc_desc(psc));
                goto bye;
             }
@@ -715,7 +718,7 @@ static void i2c_check_bus(I2C_Bus_Info * bus_info) {
                psc = i2c_get_parsed_edid_by_fd(file, &bus_info->edid);
                if (psc != 0) {
                   DBGMSF(debug, "i2c_get_parsed_edid_by_fd() returned %d", psc);
-                  f0printf(FERR, "Failure getting EDID for /dev/i2c-%d: status code=%s\n",
+                  f0printf(ferr(), "Failure getting EDID for /dev/i2c-%d: status code=%s\n",
                                  bus_info->busno, psc_desc(psc));
                   goto bye;
                }
@@ -923,7 +926,6 @@ int i2c_detect_buses() {
    bool debug = false;
    DBGMSF(debug, "Starting.  i2c_buses = %p", i2c_buses);
    if (!i2c_buses) {
-
       Byte_Value_Array i2c_bus_bva = get_i2c_device_numbers_using_udev(false);
       // TODO: set free function
       i2c_buses = g_ptr_array_sized_new(bva_length(i2c_bus_bva));
@@ -937,6 +939,7 @@ int i2c_detect_buses() {
             i2c_dbgrpt_bus_info(businfo, 0);
          g_ptr_array_add(i2c_buses, businfo);
       }
+      bva_free(i2c_bus_bva);
    }
    int result = i2c_buses->len;
    DBGMSF(debug, "Returning: %d", result);
@@ -1276,10 +1279,10 @@ bool i2c_is_valid_bus(int busno, Call_Options callopts) {
       result = true;
 
    if (complaint && emit_error_msg) {
-      f0printf(FERR, "%s /dev/i2c-%d\n", complaint, busno);
+      f0printf(ferr(), "%s /dev/i2c-%d\n", complaint, busno);
    }
    if (complaint && overridable && (callopts & CALLOPT_FORCE)) {
-      f0printf(FERR, "Continuing.  --force option was specified.\n");
+      f0printf(ferr(), "Continuing.  --force option was specified.\n");
       result = true;
    }
 
