@@ -497,9 +497,11 @@ strsplit_maxlength(
       start = start + strlen(piece);
    }
 
-   Null_Terminated_String_Array result =
-          g_ptr_array_to_ntsa(pieces);
-   g_ptr_array_free(pieces, false);
+   // for some reason, if g_ptr_array_to_ntsa() is called with duplicate=false and
+   // g_ptr_array(pieces, false) is called, valgrind complains about memory leak
+   Null_Terminated_String_Array result = g_ptr_array_to_ntsa(pieces, /*duplicate=*/ true);
+   g_ptr_array_set_free_func(pieces, g_free);
+   g_ptr_array_free(pieces, true);
    free(str_to_split2);
    if (debug)
       ntsa_show(result);
@@ -650,17 +652,25 @@ GPtrArray * ntsa_to_g_ptr_array(Null_Terminated_String_Array ntsa) {
 }
 
 
-/** Converts a GPtrArray if pointers to strings to a Null_Terminated_String_Array.
- * The underlying strings are referenced, not duplicated.
+/** Converts a GPtrArray of pointers to strings to a Null_Terminated_String_Array.
  *
  * @param gparray pointer to GPtrArray
+ * @param duplicate if true, the strings are duplicated
+ *                  if false, the pointers are copied
  * @return null-terminated array of string pointers
  */
-Null_Terminated_String_Array g_ptr_array_to_ntsa(GPtrArray * gparray) {
+Null_Terminated_String_Array
+g_ptr_array_to_ntsa(
+      GPtrArray * gparray,
+      bool        duplicate)
+{
    assert(gparray);
    Null_Terminated_String_Array ntsa = calloc(gparray->len+1, sizeof(char *));
    for (int ndx=0; ndx < gparray->len; ndx++) {
-      ntsa[ndx] = g_ptr_array_index(gparray,ndx);
+      if (duplicate)
+         ntsa[ndx] = strdup(g_ptr_array_index(gparray,ndx));
+      else
+         ntsa[ndx] = g_ptr_array_index(gparray,ndx);
    }
    return ntsa;
 }
@@ -799,6 +809,53 @@ bool sbuf_append(char * buf, int bufsz, char * sepstr, char * nextval) {
 // Integer conversion
 //
 
+/** Converts a decimal or hexadecimal string to an integer value.
+ *
+ * @param sval   string representing an integer
+ * @param p_ival address at which to store integer value
+ * @param base   10, 16, or 0 (see below)
+ * @return true if conversion succeeded, false if it failed
+ *
+ * @remark
+ * If base=10, this a normal integer conversion.
+ * If base=16, **sval** contains a hex value, optionally
+ * beginning with "x" or "0x".
+ * If base=0, then either a decimal or hexadecimal conversion is performed.
+ * If the value begins with "x" or "0x", it is treated as a hex value.
+ * Otherwise it is treated as a decimal value.
+ * \remark
+ * If conversion fails, the value pointed to by **p_ival** is unchanged.
+ * @remark
+ * This function wraps system function strtol(), hiding the ugly details.
+ */
+bool str_to_int(const char * sval, int * p_ival, int base) {
+   assert (base == 0 || base == 10 || base == 16);
+   bool debug = false;
+   if (debug)
+      printf("(%s) sval->|%s|\n", __func__, sval);
+
+   char * endptr;
+   bool ok = false;
+   if ( *sval != '\0') {
+      long result = strtol(sval, &endptr, base); // allow hex
+      // printf("(%s) sval=%p, endptr=%p, *endptr=|%c| (0x%02x), result=%ld\n",
+      //        __func__, sval, endptr, *endptr, *endptr, result);
+      if (*endptr == '\0') {
+         *p_ival = result;
+         ok = true;
+      }
+   }
+
+   if (debug) {
+      if (ok)
+        printf("(%s) sval=%s, Returning: %s, *ival = %d\n", __func__, sval, bool_repr(ok), *p_ival);
+      else
+        printf("(%s) sval=%s, Returning: %s\n", __func__, sval, bool_repr(ok));
+   }
+   return ok;
+}
+
+#ifdef OLD
 /** Converts a string representing an integer to an integer value.
  *
  * @param sval   string representing an integer
@@ -810,7 +867,7 @@ bool sbuf_append(char * buf, int bufsz, char * sepstr, char * nextval) {
  * @remark
  * This function wraps system function strtol(), hiding the ugly details.
  */
-bool str_to_int(const char * sval, int * p_ival) {
+bool str_to_int_old(const char * sval, int * p_ival) {
    bool debug = false;
    if (debug)
       printf("(%s) sval->|%s|\n", __func__, sval);
@@ -835,6 +892,7 @@ bool str_to_int(const char * sval, int * p_ival) {
    }
    return ok;
 }
+#endif
 
 
 //
@@ -1357,7 +1415,7 @@ void fhex_dump_indented(FILE * fh, const Byte* data, int size, int indents)
    if (fh) {
       int i; // index in data...
       int j; // index in line...
-      char temp[8];
+      char temp[10];    // was 8, compiler complains that too small
       char buffer[128];
       char *ascii;
       char indentation[100];
@@ -1534,10 +1592,18 @@ char * ascii_strcasestr(const char * haystack, const char * needle) {
 }
 
 
-// Belongs in some more general file.   where?  string_util.c, data_structures.c?
-// Idea: allow for option to treat terms as regular expressions
-// convert parm ignore_case into a flags byte of options
-
+/** Tests if any of a set of strings is a substring of a given string
+ *
+ *  @param test         string to check, must not be NULL
+ *  @param terms        pointer to array of substring to check
+ *  @param ignore_case  if true, test is case-insensitive
+ *  @return true if a substring is found
+ *
+ *  @remark
+ *  If **terms** is NULL, returns **false**
+ *  @remark
+ *  Consider converting ignore_case to a flags byte of options
+ */
 bool apply_filter_terms(const char * text, char ** terms, bool ignore_case) {
    assert(text);
    bool debug = false;
